#include <cv.h>
#include <ch.h>

EXPORTCH void* cvAlloc_chdl(void *varg) {
    va_list ap;
    size_t size;
    void* retval;

    Ch_VaStart(ap, varg);
    size = Ch_VaArg(ap, size_t);
    retval = cvAlloc(size);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvFree_chdl(void *varg) {
    va_list ap;
    void** ptr;

    Ch_VaStart(ap, varg);
    ptr = Ch_VaArg(ap, void**);
    cvFree(ptr);
    Ch_VaEnd(ap);
}

EXPORTCH IplImage* cvCreateImageHeader_chdl(void *varg) {
    va_list ap;
    CvSize size;
    int depth;
    int channels;
    IplImage* retval;

    Ch_VaStart(ap, varg);
    size = Ch_VaArg(ap, CvSize);
    depth = Ch_VaArg(ap, int);
    channels = Ch_VaArg(ap, int);
    retval = cvCreateImageHeader(size, depth, channels);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH IplImage* cvInitImageHeader_chdl(void *varg) {
    va_list ap;
    IplImage* image;
    CvSize size;
    int depth;
    int channels;
    int origin;
    int align;
    IplImage* retval;

    Ch_VaStart(ap, varg);
    image = Ch_VaArg(ap, IplImage*);
    size = Ch_VaArg(ap, CvSize);
    depth = Ch_VaArg(ap, int);
    channels = Ch_VaArg(ap, int);
    origin = Ch_VaArg(ap, int);
    align = Ch_VaArg(ap, int);
    retval = cvInitImageHeader(image, size, depth, channels, origin, align);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH IplImage* cvCreateImage_chdl(void *varg) {
    va_list ap;
    CvSize size;
    int depth;
    int channels;
    IplImage* retval;

    Ch_VaStart(ap, varg);
    size = Ch_VaArg(ap, CvSize);
    depth = Ch_VaArg(ap, int);
    channels = Ch_VaArg(ap, int);
    retval = cvCreateImage(size, depth, channels);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvReleaseImageHeader_chdl(void *varg) {
    va_list ap;
    IplImage** image;

    Ch_VaStart(ap, varg);
    image = Ch_VaArg(ap, IplImage**);
    cvReleaseImageHeader(image);
    Ch_VaEnd(ap);
}

EXPORTCH void cvReleaseImage_chdl(void *varg) {
    va_list ap;
    IplImage** image;

    Ch_VaStart(ap, varg);
    image = Ch_VaArg(ap, IplImage**);
    cvReleaseImage(image);
    Ch_VaEnd(ap);
}

EXPORTCH IplImage* cvCloneImage_chdl(void *varg) {
    va_list ap;
    const IplImage* image;
    IplImage* retval;

    Ch_VaStart(ap, varg);
    image = Ch_VaArg(ap, const IplImage*);
    retval = cvCloneImage(image);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvSetImageCOI_chdl(void *varg) {
    va_list ap;
    IplImage* image;
    int coi;

    Ch_VaStart(ap, varg);
    image = Ch_VaArg(ap, IplImage*);
    coi = Ch_VaArg(ap, int);
    cvSetImageCOI(image, coi);
    Ch_VaEnd(ap);
}

EXPORTCH int cvGetImageCOI_chdl(void *varg) {
    va_list ap;
    IplImage* image;
    int retval;

    Ch_VaStart(ap, varg);
    image = Ch_VaArg(ap, IplImage*);
    retval = cvGetImageCOI(image);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvSetImageROI_chdl(void *varg) {
    va_list ap;
    IplImage* image;
    CvRect rect;

    Ch_VaStart(ap, varg);
    image = Ch_VaArg(ap, IplImage*);
    rect = Ch_VaArg(ap, CvRect);
    cvSetImageROI(image, rect);
    Ch_VaEnd(ap);
}

EXPORTCH void cvResetImageROI_chdl(void *varg) {
    va_list ap;
    IplImage* image;

    Ch_VaStart(ap, varg);
    image = Ch_VaArg(ap, IplImage*);
    cvResetImageROI(image);
    Ch_VaEnd(ap);
}

/*
EXPORTCH CvRect cvGetImageROI_chdl(void *varg) {
    va_list ap;
    const IplImage* image;
    CvRect retval;

    Ch_VaStart(ap, varg);
    image = Ch_VaArg(ap, const IplImage*);
    retval = cvGetImageROI(image);
    Ch_VaEnd(ap);
    return retval;
}

*/
EXPORTCH CvMat* cvCreateMatHeader_chdl(void *varg) {
    va_list ap;
    int rows;
    int cols;
    int type ;
    CvMat* retval;

    Ch_VaStart(ap, varg);
    rows = Ch_VaArg(ap, int);
    cols = Ch_VaArg(ap, int);
    type  = Ch_VaArg(ap, int);
    retval = cvCreateMatHeader(rows, cols, type );
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CvMat* cvInitMatHeader_chdl(void *varg) {
    va_list ap;
    CvMat* mat;
    int rows;
    int cols;
    int type;
    void* data;
    int step;
    CvMat* retval;

    Ch_VaStart(ap, varg);
    mat = Ch_VaArg(ap, CvMat*);
    rows = Ch_VaArg(ap, int);
    cols = Ch_VaArg(ap, int);
    type = Ch_VaArg(ap, int);
    data = Ch_VaArg(ap, void*);
    step = Ch_VaArg(ap, int);
    retval = cvInitMatHeader(mat, rows, cols, type, data, step);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CvMat* cvCreateMat_chdl(void *varg) {
    va_list ap;
    int rows;
    int cols;
    int type ;
    CvMat* retval;

    Ch_VaStart(ap, varg);
    rows = Ch_VaArg(ap, int);
    cols = Ch_VaArg(ap, int);
    type  = Ch_VaArg(ap, int);
    retval = cvCreateMat(rows, cols, type );
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvReleaseMat_chdl(void *varg) {
    va_list ap;
    CvMat** mat;

    Ch_VaStart(ap, varg);
    mat = Ch_VaArg(ap, CvMat**);
    cvReleaseMat(mat);
    Ch_VaEnd(ap);
}

EXPORTCH CvMat* cvCloneMat_chdl(void *varg) {
    va_list ap;
    const CvMat* mat;
    CvMat* retval;

    Ch_VaStart(ap, varg);
    mat = Ch_VaArg(ap, const CvMat*);
    retval = cvCloneMat(mat);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CvMat* cvGetSubRect_chdl(void *varg) {
    va_list ap;
    const CvArr* arr;
    CvMat* submat;
    CvRect rect ;
    CvMat* retval;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, const CvArr*);
    submat = Ch_VaArg(ap, CvMat*);
    rect  = Ch_VaArg(ap, CvRect);
    retval = cvGetSubRect(arr, submat, rect );
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CvMat* cvGetRows_chdl(void *varg) {
    va_list ap;
    const CvArr* arr;
    CvMat* submat;
    int start_row;
    int end_row;
    CvMat* retval;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, const CvArr*);
    submat = Ch_VaArg(ap, CvMat*);
    start_row = Ch_VaArg(ap, int);
    end_row = Ch_VaArg(ap, int);
    retval = cvGetRows(arr, submat, start_row, end_row);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CvMat* cvGetCols_chdl(void *varg) {
    va_list ap;
    const CvArr* arr;
    CvMat* submat;
    int start_col;
    int end_col;
    CvMat* retval;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, const CvArr*);
    submat = Ch_VaArg(ap, CvMat*);
    start_col = Ch_VaArg(ap, int);
    end_col = Ch_VaArg(ap, int);
    retval = cvGetCols(arr, submat, start_col, end_col);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CvMat* cvGetDiag_chdl(void *varg) {
    va_list ap;
    const CvArr* arr;
    CvMat* submat;
    int diag;
    CvMat* retval;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, const CvArr*);
    submat = Ch_VaArg(ap, CvMat*);
    diag = Ch_VaArg(ap, int);
    retval = cvGetDiag(arr, submat, diag);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvScalarToRawData_chdl(void *varg) {
    va_list ap;
    const CvScalar* scalar;
    void* data;
    int type;
    int extend_to_12;

    Ch_VaStart(ap, varg);
    scalar = Ch_VaArg(ap, const CvScalar*);
    data = Ch_VaArg(ap, void*);
    type = Ch_VaArg(ap, int);
    extend_to_12 = Ch_VaArg(ap, int);
    cvScalarToRawData(scalar, data, type, extend_to_12);
    Ch_VaEnd(ap);
}

EXPORTCH void cvRawDataToScalar_chdl(void *varg) {
    va_list ap;
    const void* data;
    int type;
    CvScalar* scalar;

    Ch_VaStart(ap, varg);
    data = Ch_VaArg(ap, const void*);
    type = Ch_VaArg(ap, int);
    scalar = Ch_VaArg(ap, CvScalar*);
    cvRawDataToScalar(data, type, scalar);
    Ch_VaEnd(ap);
}

EXPORTCH CvMatND* cvCreateMatNDHeader_chdl(void *varg) {
    va_list ap;
    int dims;
    const int* sizes;
    int type;
    CvMatND* retval;

    Ch_VaStart(ap, varg);
    dims = Ch_VaArg(ap, int);
    sizes = Ch_VaArg(ap, const int*);
    type = Ch_VaArg(ap, int);
    retval = cvCreateMatNDHeader(dims, sizes, type);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CvMatND* cvCreateMatND_chdl(void *varg) {
    va_list ap;
    int dims;
    const int* sizes;
    int type;
    CvMatND* retval;

    Ch_VaStart(ap, varg);
    dims = Ch_VaArg(ap, int);
    sizes = Ch_VaArg(ap, const int*);
    type = Ch_VaArg(ap, int);
    retval = cvCreateMatND(dims, sizes, type);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CvMatND* cvInitMatNDHeader_chdl(void *varg) {
    va_list ap;
    CvMatND* mat;
    int dims;
    const int* sizes;
    int type;
    void* data;
    CvMatND* retval;

    Ch_VaStart(ap, varg);
    mat = Ch_VaArg(ap, CvMatND*);
    dims = Ch_VaArg(ap, int);
    sizes = Ch_VaArg(ap, const int*);
    type = Ch_VaArg(ap, int);
    data = Ch_VaArg(ap, void*);
    retval = cvInitMatNDHeader(mat, dims, sizes, type, data);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CvMatND* cvCloneMatND_chdl(void *varg) {
    va_list ap;
    const CvMatND* mat ;
    CvMatND* retval;

    Ch_VaStart(ap, varg);
    mat  = Ch_VaArg(ap, const CvMatND*);
    retval = cvCloneMatND(mat );
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CvSparseMat* cvCreateSparseMat_chdl(void *varg) {
    va_list ap;
    int dims;
    const int* sizes;
    int type;
    CvSparseMat* retval;

    Ch_VaStart(ap, varg);
    dims = Ch_VaArg(ap, int);
    sizes = Ch_VaArg(ap, const int*);
    type = Ch_VaArg(ap, int);
    retval = cvCreateSparseMat(dims, sizes, type);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvReleaseSparseMat_chdl(void *varg) {
    va_list ap;
    CvSparseMat** mat ;

    Ch_VaStart(ap, varg);
    mat  = Ch_VaArg(ap, CvSparseMat**);
    cvReleaseSparseMat(mat );
    Ch_VaEnd(ap);
}

EXPORTCH CvSparseMat* cvCloneSparseMat_chdl(void *varg) {
    va_list ap;
    const CvSparseMat* mat ;
    CvSparseMat* retval;

    Ch_VaStart(ap, varg);
    mat  = Ch_VaArg(ap, const CvSparseMat*);
    retval = cvCloneSparseMat(mat );
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CvSparseNode* cvInitSparseMatIterator_chdl(void *varg) {
    va_list ap;
    const CvSparseMat* mat;
    CvSparseMatIterator* matIterator;
    CvSparseNode* retval;

    Ch_VaStart(ap, varg);
    mat = Ch_VaArg(ap, const CvSparseMat*);
    matIterator = Ch_VaArg(ap, CvSparseMatIterator*);
    retval = cvInitSparseMatIterator(mat, matIterator);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH int cvGetElemType_chdl(void *varg) {
    va_list ap;
    const CvArr* arr ;
    int retval;

    Ch_VaStart(ap, varg);
    arr  = Ch_VaArg(ap, const CvArr*);
    retval = cvGetElemType(arr );
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH int cvGetDims_chdl(void *varg) {
    va_list ap;
    const CvArr* arr;
    int* sizes;
    int retval;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, const CvArr*);
    sizes = Ch_VaArg(ap, int*);
    retval = cvGetDims(arr, sizes);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH int cvGetDimSize_chdl(void *varg) {
    va_list ap;
    const CvArr* arr;
    int index;
    int retval;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, const CvArr*);
    index = Ch_VaArg(ap, int);
    retval = cvGetDimSize(arr, index);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH uchar* cvPtr1D_chdl(void *varg) {
    va_list ap;
    const CvArr* arr;
    int idx1;
    int* type;
    uchar* retval;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, const CvArr*);
    idx1 = Ch_VaArg(ap, int);
    type = Ch_VaArg(ap, int*);
    retval = cvPtr1D(arr, idx1, type);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH uchar* cvPtr2D_chdl(void *varg) {
    va_list ap;
    const CvArr* arr;
    int idx1;
    int idx2;
    int* type;
    uchar* retval;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, const CvArr*);
    idx1 = Ch_VaArg(ap, int);
    idx2 = Ch_VaArg(ap, int);
    type = Ch_VaArg(ap, int*);
    retval = cvPtr2D(arr, idx1, idx2, type);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH uchar* cvPtr3D_chdl(void *varg) {
    va_list ap;
    const CvArr* arr;
    int idx1;
    int idx2;
    int idx3;
    int* type;
    uchar* retval;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, const CvArr*);
    idx1 = Ch_VaArg(ap, int);
    idx2 = Ch_VaArg(ap, int);
    idx3 = Ch_VaArg(ap, int);
    type = Ch_VaArg(ap, int*);
    retval = cvPtr3D(arr, idx1, idx2, idx3, type);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH uchar* cvPtrND_chdl(void *varg) {
    va_list ap;
    const CvArr* arr;
    int* idx;
    int* type;
    uchar* retval;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, const CvArr*);
    idx = Ch_VaArg(ap, int*);
    type = Ch_VaArg(ap, int*);
    retval = cvPtrND(arr, idx, type);
    Ch_VaEnd(ap);
    return retval;
}

/*
EXPORTCH CvScalar cvGet1D_chdl(void *varg) {
    va_list ap;
    const CvArr* arr;
    int idx1;
    CvScalar retval;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, const CvArr*);
    idx1 = Ch_VaArg(ap, int);
    retval = cvGet1D(arr, idx1);
    Ch_VaEnd(ap);
    return retval;
}

*/
/*
EXPORTCH CvScalar cvGet2D_chdl(void *varg) {
    va_list ap;
    const CvArr* arr;
    int idx1;
    int idx2;
    CvScalar retval;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, const CvArr*);
    idx1 = Ch_VaArg(ap, int);
    idx2 = Ch_VaArg(ap, int);
    retval = cvGet2D(arr, idx1, idx2);
    Ch_VaEnd(ap);
    return retval;
}

*/
/*
EXPORTCH CvScalar cvGet3D_chdl(void *varg) {
    va_list ap;
    const CvArr* arr;
    int idx1;
    int idx2;
    int idx3;
    CvScalar retval;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, const CvArr*);
    idx1 = Ch_VaArg(ap, int);
    idx2 = Ch_VaArg(ap, int);
    idx3 = Ch_VaArg(ap, int);
    retval = cvGet3D(arr, idx1, idx2, idx3);
    Ch_VaEnd(ap);
    return retval;
}

*/
/*
EXPORTCH CvScalar cvGetND_chdl(void *varg) {
    va_list ap;
    const CvArr* arr;
    int* idx;
    CvScalar retval;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, const CvArr*);
    idx = Ch_VaArg(ap, int*);
    retval = cvGetND(arr, idx);
    Ch_VaEnd(ap);
    return retval;
}

*/
EXPORTCH double cvGetReal1D_chdl(void *varg) {
    va_list ap;
    const CvArr* arr;
    int idx1;
    double retval;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, const CvArr*);
    idx1 = Ch_VaArg(ap, int);
    retval = cvGetReal1D(arr, idx1);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH double cvGetReal2D_chdl(void *varg) {
    va_list ap;
    const CvArr* arr;
    int idx1;
    int idx2;
    double retval;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, const CvArr*);
    idx1 = Ch_VaArg(ap, int);
    idx2 = Ch_VaArg(ap, int);
    retval = cvGetReal2D(arr, idx1, idx2);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH double cvGetReal3D_chdl(void *varg) {
    va_list ap;
    const CvArr* arr;
    int idx1;
    int idx2;
    int idx3;
    double retval;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, const CvArr*);
    idx1 = Ch_VaArg(ap, int);
    idx2 = Ch_VaArg(ap, int);
    idx3 = Ch_VaArg(ap, int);
    retval = cvGetReal3D(arr, idx1, idx2, idx3);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH double cvGetRealND_chdl(void *varg) {
    va_list ap;
    const CvArr* arr;
    int* idx;
    double retval;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, const CvArr*);
    idx = Ch_VaArg(ap, int*);
    retval = cvGetRealND(arr, idx);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvSet1D_chdl(void *varg) {
    va_list ap;
    CvArr* arr;
    int idx1;
    CvScalar value;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, CvArr*);
    idx1 = Ch_VaArg(ap, int);
    value = Ch_VaArg(ap, CvScalar);
    cvSet1D(arr, idx1, value);
    Ch_VaEnd(ap);
}

EXPORTCH void cvSet2D_chdl(void *varg) {
    va_list ap;
    CvArr* arr;
    int idx1;
    int idx2;
    CvScalar value;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, CvArr*);
    idx1 = Ch_VaArg(ap, int);
    idx2 = Ch_VaArg(ap, int);
    value = Ch_VaArg(ap, CvScalar);
    cvSet2D(arr, idx1, idx2, value);
    Ch_VaEnd(ap);
}

EXPORTCH void cvSet3D_chdl(void *varg) {
    va_list ap;
    CvArr* arr;
    int idx1;
    int idx2;
    int idx3;
    CvScalar value;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, CvArr*);
    idx1 = Ch_VaArg(ap, int);
    idx2 = Ch_VaArg(ap, int);
    idx3 = Ch_VaArg(ap, int);
    value = Ch_VaArg(ap, CvScalar);
    cvSet3D(arr, idx1, idx2, idx3, value);
    Ch_VaEnd(ap);
}

EXPORTCH void cvSetND_chdl(void *varg) {
    va_list ap;
    CvArr* arr;
    int* idx;
    CvScalar value;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, CvArr*);
    idx = Ch_VaArg(ap, int*);
    value = Ch_VaArg(ap, CvScalar);
    cvSetND(arr, idx, value);
    Ch_VaEnd(ap);
}

EXPORTCH void cvSetReal1D_chdl(void *varg) {
    va_list ap;
    CvArr* arr;
    int idx1;
    double value;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, CvArr*);
    idx1 = Ch_VaArg(ap, int);
    value = Ch_VaArg(ap, double);
    cvSetReal1D(arr, idx1, value);
    Ch_VaEnd(ap);
}

EXPORTCH void cvSetReal2D_chdl(void *varg) {
    va_list ap;
    CvArr* arr;
    int idx1;
    int idx2;
    double value;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, CvArr*);
    idx1 = Ch_VaArg(ap, int);
    idx2 = Ch_VaArg(ap, int);
    value = Ch_VaArg(ap, double);
    cvSetReal2D(arr, idx1, idx2, value);
    Ch_VaEnd(ap);
}

EXPORTCH void cvSetReal3D_chdl(void *varg) {
    va_list ap;
    CvArr* arr;
    int idx1;
    int idx2;
    int idx3;
    double value;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, CvArr*);
    idx1 = Ch_VaArg(ap, int);
    idx2 = Ch_VaArg(ap, int);
    idx3 = Ch_VaArg(ap, int);
    value = Ch_VaArg(ap, double);
    cvSetReal3D(arr, idx1, idx2, idx3, value);
    Ch_VaEnd(ap);
}

EXPORTCH void cvSetRealND_chdl(void *varg) {
    va_list ap;
    CvArr* arr;
    int* idx;
    double value;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, CvArr*);
    idx = Ch_VaArg(ap, int*);
    value = Ch_VaArg(ap, double);
    cvSetRealND(arr, idx, value);
    Ch_VaEnd(ap);
}

EXPORTCH void cvClearND_chdl(void *varg) {
    va_list ap;
    CvArr* arr;
    int* idx;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, CvArr*);
    idx = Ch_VaArg(ap, int*);
    cvClearND(arr, idx);
    Ch_VaEnd(ap);
}

EXPORTCH CvMat* cvGetMat_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvMat* header;
    int* coi;
    int allowND;
    CvMat* retval;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    header = Ch_VaArg(ap, CvMat*);
    coi = Ch_VaArg(ap, int*);
    allowND = Ch_VaArg(ap, int);
    retval = cvGetMat(src, header, coi, allowND);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH IplImage* cvGetImage_chdl(void *varg) {
    va_list ap;
    const CvArr* arr;
    IplImage* img ;
    IplImage* retval;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, const CvArr*);
    img  = Ch_VaArg(ap, IplImage*);
    retval = cvGetImage(arr, img );
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CvArr* cvReshapeMatND_chdl(void *varg) {
    va_list ap;
    const CvArr* arr;
    int sizeof_header;
    CvArr* header;
    int new_cn;
    int new_dims;
    int* new_sizes;
    CvArr* retval;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, const CvArr*);
    sizeof_header = Ch_VaArg(ap, int);
    header = Ch_VaArg(ap, CvArr*);
    new_cn = Ch_VaArg(ap, int);
    new_dims = Ch_VaArg(ap, int);
    new_sizes = Ch_VaArg(ap, int*);
    retval = cvReshapeMatND(arr, sizeof_header, header, new_cn, new_dims, new_sizes);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CvMat* cvReshape_chdl(void *varg) {
    va_list ap;
    const CvArr* arr;
    CvMat* header;
    int new_cn;
    int new_rows;
    CvMat* retval;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, const CvArr*);
    header = Ch_VaArg(ap, CvMat*);
    new_cn = Ch_VaArg(ap, int);
    new_rows = Ch_VaArg(ap, int);
    retval = cvReshape(arr, header, new_cn, new_rows);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvRepeat_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvArr* dst ;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    dst  = Ch_VaArg(ap, CvArr*);
    cvRepeat(src, dst );
    Ch_VaEnd(ap);
}

EXPORTCH void cvCreateData_chdl(void *varg) {
    va_list ap;
    CvArr* arr ;

    Ch_VaStart(ap, varg);
    arr  = Ch_VaArg(ap, CvArr*);
    cvCreateData(arr );
    Ch_VaEnd(ap);
}

EXPORTCH void cvReleaseData_chdl(void *varg) {
    va_list ap;
    CvArr* arr ;

    Ch_VaStart(ap, varg);
    arr  = Ch_VaArg(ap, CvArr*);
    cvReleaseData(arr );
    Ch_VaEnd(ap);
}

EXPORTCH void cvSetData_chdl(void *varg) {
    va_list ap;
    CvArr* arr;
    void* data;
    int step;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, CvArr*);
    data = Ch_VaArg(ap, void*);
    step = Ch_VaArg(ap, int);
    cvSetData(arr, data, step);
    Ch_VaEnd(ap);
}

EXPORTCH void cvGetRawData_chdl(void *varg) {
    va_list ap;
    const CvArr* arr;
    uchar** data;
    int* step;
    CvSize* roi_size;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, const CvArr*);
    data = Ch_VaArg(ap, uchar**);
    step = Ch_VaArg(ap, int*);
    roi_size = Ch_VaArg(ap, CvSize*);
    cvGetRawData(arr, data, step, roi_size);
    Ch_VaEnd(ap);
}

/*
EXPORTCH CvSize cvGetSize_chdl(void *varg) {
    va_list ap;
    const CvArr* arr ;
    CvSize retval;

    Ch_VaStart(ap, varg);
    arr  = Ch_VaArg(ap, const CvArr*);
    retval = cvGetSize(arr );
    Ch_VaEnd(ap);
    return retval;
}

*/
EXPORTCH void cvCopy_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvArr* dst;
    const CvArr* mask;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    mask = Ch_VaArg(ap, const CvArr*);
    cvCopy(src, dst, mask);
    Ch_VaEnd(ap);
}

EXPORTCH void cvSet_chdl(void *varg) {
    va_list ap;
    CvArr* arr;
    CvScalar scalar;
    const CvArr* mask;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, CvArr*);
    scalar = Ch_VaArg(ap, CvScalar);
    mask = Ch_VaArg(ap, const CvArr*);
    cvSet(arr, scalar, mask);
    Ch_VaEnd(ap);
}

EXPORTCH void cvSetZero_chdl(void *varg) {
    va_list ap;
    CvArr* mat ;

    Ch_VaStart(ap, varg);
    mat  = Ch_VaArg(ap, CvArr*);
    cvSetZero(mat );
    Ch_VaEnd(ap);
}

EXPORTCH void cvCvtPixToPlane_chdl(void *varg) {
    va_list ap;
    const void *src;
    void *dst0;
    void *dst1;
    void *dst2;
    void *dst3 ;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const void *);
    dst0 = Ch_VaArg(ap, void *);
    dst1 = Ch_VaArg(ap, void *);
    dst2 = Ch_VaArg(ap, void *);
    dst3  = Ch_VaArg(ap, void *);
    cvCvtPixToPlane(src, dst0, dst1, dst2, dst3 );
    Ch_VaEnd(ap);
}

EXPORTCH void cvCvtPlaneToPix_chdl(void *varg) {
    va_list ap;
    const void *src0;
    const void *src1;
    const void *src2;
    const void *src3;
    void *dst;

    Ch_VaStart(ap, varg);
    src0 = Ch_VaArg(ap, const void *);
    src1 = Ch_VaArg(ap, const void *);
    src2 = Ch_VaArg(ap, const void *);
    src3 = Ch_VaArg(ap, const void *);
    dst = Ch_VaArg(ap, void *);
    cvCvtPlaneToPix(src0, src1, src2, src3, dst);
    Ch_VaEnd(ap);
}

EXPORTCH void cvConvertScale_chdl(void *varg) {
    va_list ap;
    const CvArr *src;
    CvArr *dst;
    double scale;
    double shift;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr *);
    dst = Ch_VaArg(ap, CvArr *);
    scale = Ch_VaArg(ap, double);
    shift = Ch_VaArg(ap, double);
    cvConvertScale(src, dst, scale, shift);
    Ch_VaEnd(ap);
}

EXPORTCH void cvConvertScaleAbs_chdl(void *varg) {
    va_list ap;
    const void *src;
    void *dst;
    double scale;
    double shift;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const void *);
    dst = Ch_VaArg(ap, void *);
    scale = Ch_VaArg(ap, double);
    shift = Ch_VaArg(ap, double);
    cvConvertScaleAbs(src, dst, scale, shift);
    Ch_VaEnd(ap);
}

EXPORTCH CvRect cvMaxRect_chdl(void *varg) {
    va_list ap;
    const CvRect* rect1;
    const CvRect* rect2 ;
    CvRect retval;

    Ch_VaStart(ap, varg);
    rect1 = Ch_VaArg(ap, const CvRect*);
    rect2  = Ch_VaArg(ap, const CvRect*);
    retval = cvMaxRect(rect1, rect2 );
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvBoxPoints_chdl(void *varg) {
    va_list ap;
    CvBox2D box;
    CvPoint2D32f *pt;

    Ch_VaStart(ap, varg);
    box = Ch_VaArg(ap, CvBox2D);
    pt = Ch_VaArg(ap, CvPoint2D32f *);
    cvBoxPoints(box, pt);
    Ch_VaEnd(ap);
}

EXPORTCH void cvAdd_chdl(void *varg) {
    va_list ap;
    const CvArr* srcA;
    const CvArr* srcB;
    CvArr* dst;
    const CvArr* mask;

    Ch_VaStart(ap, varg);
    srcA = Ch_VaArg(ap, const CvArr*);
    srcB = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    mask = Ch_VaArg(ap, const CvArr*);
    cvAdd(srcA, srcB, dst, mask);
    Ch_VaEnd(ap);
}

EXPORTCH void cvAddS_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvScalar value;
    CvArr* dst;
    const CvArr* mask;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    value = Ch_VaArg(ap, CvScalar);
    dst = Ch_VaArg(ap, CvArr*);
    mask = Ch_VaArg(ap, const CvArr*);
    cvAddS(src, value, dst, mask);
    Ch_VaEnd(ap);
}

EXPORTCH void cvSub_chdl(void *varg) {
    va_list ap;
    const CvArr* srcA;
    const CvArr* srcB;
    CvArr* dst;
    const CvArr* mask;

    Ch_VaStart(ap, varg);
    srcA = Ch_VaArg(ap, const CvArr*);
    srcB = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    mask = Ch_VaArg(ap, const CvArr*);
    cvSub(srcA, srcB, dst, mask);
    Ch_VaEnd(ap);
}

EXPORTCH void cvSubRS_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvScalar value;
    CvArr* dst;
    const CvArr* mask;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    value = Ch_VaArg(ap, CvScalar);
    dst = Ch_VaArg(ap, CvArr*);
    mask = Ch_VaArg(ap, const CvArr*);
    cvSubRS(src, value, dst, mask);
    Ch_VaEnd(ap);
}

EXPORTCH void cvMul_chdl(void *varg) {
    va_list ap;
    const CvArr* srcA;
    const CvArr* srcB;
    CvArr* dst;
    double scale;

    Ch_VaStart(ap, varg);
    srcA = Ch_VaArg(ap, const CvArr*);
    srcB = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    scale = Ch_VaArg(ap, double);
    cvMul(srcA, srcB, dst, scale);
    Ch_VaEnd(ap);
}

EXPORTCH void cvDiv_chdl(void *varg) {
    va_list ap;
    const CvArr* srcA;
    const CvArr* srcB;
    CvArr* dst;
    double scale;

    Ch_VaStart(ap, varg);
    srcA = Ch_VaArg(ap, const CvArr*);
    srcB = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    scale = Ch_VaArg(ap, double);
    cvDiv(srcA, srcB, dst, scale);
    Ch_VaEnd(ap);
}

EXPORTCH void cvScaleAdd_chdl(void *varg) {
    va_list ap;
    const CvArr* srcA;
    CvScalar scale;
    const CvArr* srcB;
    CvArr* dst;

    Ch_VaStart(ap, varg);
    srcA = Ch_VaArg(ap, const CvArr*);
    scale = Ch_VaArg(ap, CvScalar);
    srcB = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    cvScaleAdd(srcA, scale, srcB, dst);
    Ch_VaEnd(ap);
}

EXPORTCH void cvAddWeighted_chdl(void *varg) {
    va_list ap;
    const CvArr* srcA;
    double alpha;
    const CvArr* srcB;
    double beta;
    double gamma;
    CvArr* dst ;

    Ch_VaStart(ap, varg);
    srcA = Ch_VaArg(ap, const CvArr*);
    alpha = Ch_VaArg(ap, double);
    srcB = Ch_VaArg(ap, const CvArr*);
    beta = Ch_VaArg(ap, double);
    gamma = Ch_VaArg(ap, double);
    dst  = Ch_VaArg(ap, CvArr*);
    cvAddWeighted(srcA, alpha, srcB, beta, gamma, dst );
    Ch_VaEnd(ap);
}

EXPORTCH double cvDotProduct_chdl(void *varg) {
    va_list ap;
    const CvArr* srcA;
    const CvArr* srcB;
    double retval;

    Ch_VaStart(ap, varg);
    srcA = Ch_VaArg(ap, const CvArr*);
    srcB = Ch_VaArg(ap, const CvArr*);
    retval = cvDotProduct(srcA, srcB);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvAnd_chdl(void *varg) {
    va_list ap;
    const CvArr* src1;
    const CvArr* src2;
    CvArr* dst;
    const CvArr* mask;

    Ch_VaStart(ap, varg);
    src1 = Ch_VaArg(ap, const CvArr*);
    src2 = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    mask = Ch_VaArg(ap, const CvArr*);
    cvAnd(src1, src2, dst, mask);
    Ch_VaEnd(ap);
}

EXPORTCH void cvAndS_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvScalar value;
    CvArr* dst;
    const CvArr* mask;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    value = Ch_VaArg(ap, CvScalar);
    dst = Ch_VaArg(ap, CvArr*);
    mask = Ch_VaArg(ap, const CvArr*);
    cvAndS(src, value, dst, mask);
    Ch_VaEnd(ap);
}

EXPORTCH void cvOr_chdl(void *varg) {
    va_list ap;
    const CvArr* src1;
    const CvArr* src2;
    CvArr* dst;
    const CvArr* mask;

    Ch_VaStart(ap, varg);
    src1 = Ch_VaArg(ap, const CvArr*);
    src2 = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    mask = Ch_VaArg(ap, const CvArr*);
    cvOr(src1, src2, dst, mask);
    Ch_VaEnd(ap);
}

EXPORTCH void cvOrS_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvScalar value;
    CvArr* dst;
    const CvArr* mask;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    value = Ch_VaArg(ap, CvScalar);
    dst = Ch_VaArg(ap, CvArr*);
    mask = Ch_VaArg(ap, const CvArr*);
    cvOrS(src, value, dst, mask);
    Ch_VaEnd(ap);
}

EXPORTCH void cvXor_chdl(void *varg) {
    va_list ap;
    const CvArr* src1;
    const CvArr* src2;
    CvArr* dst;
    const CvArr* mask;

    Ch_VaStart(ap, varg);
    src1 = Ch_VaArg(ap, const CvArr*);
    src2 = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    mask = Ch_VaArg(ap, const CvArr*);
    cvXor(src1, src2, dst, mask);
    Ch_VaEnd(ap);
}

EXPORTCH void cvXorS_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvScalar value;
    CvArr* dst;
    const CvArr* mask;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    value = Ch_VaArg(ap, CvScalar);
    dst = Ch_VaArg(ap, CvArr*);
    mask = Ch_VaArg(ap, const CvArr*);
    cvXorS(src, value, dst, mask);
    Ch_VaEnd(ap);
}

EXPORTCH void cvNot_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvArr* dst;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    cvNot(src, dst);
    Ch_VaEnd(ap);
}

EXPORTCH void cvInRange_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    const CvArr* lower;
    const CvArr* upper;
    CvArr* dst;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    lower = Ch_VaArg(ap, const CvArr*);
    upper = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    cvInRange(src, lower, upper, dst);
    Ch_VaEnd(ap);
}

EXPORTCH void cvInRangeS_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvScalar lower;
    CvScalar upper;
    CvArr* dst;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    lower = Ch_VaArg(ap, CvScalar);
    upper = Ch_VaArg(ap, CvScalar);
    dst = Ch_VaArg(ap, CvArr*);
    cvInRangeS(src, lower, upper, dst);
    Ch_VaEnd(ap);
}

EXPORTCH void cvCmp_chdl(void *varg) {
    va_list ap;
    const CvArr* src1;
    const CvArr* src2;
    CvArr* dst;
    int cmpOp;

    Ch_VaStart(ap, varg);
    src1 = Ch_VaArg(ap, const CvArr*);
    src2 = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    cmpOp = Ch_VaArg(ap, int);
    cvCmp(src1, src2, dst, cmpOp);
    Ch_VaEnd(ap);
}

EXPORTCH void cvCmpS_chdl(void *varg) {
    va_list ap;
    const CvArr* src1;
    double scalar;
    CvArr* dst;
    int cmpOp;

    Ch_VaStart(ap, varg);
    src1 = Ch_VaArg(ap, const CvArr*);
    scalar = Ch_VaArg(ap, double);
    dst = Ch_VaArg(ap, CvArr*);
    cmpOp = Ch_VaArg(ap, int);
    cvCmpS(src1, scalar, dst, cmpOp);
    Ch_VaEnd(ap);
}

EXPORTCH void cvMin_chdl(void *varg) {
    va_list ap;
    const CvArr* src1;
    const CvArr* src2;
    CvArr* dst;

    Ch_VaStart(ap, varg);
    src1 = Ch_VaArg(ap, const CvArr*);
    src2 = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    cvMin(src1, src2, dst);
    Ch_VaEnd(ap);
}

EXPORTCH void cvMax_chdl(void *varg) {
    va_list ap;
    const CvArr* src1;
    const CvArr* src2;
    CvArr* dst;

    Ch_VaStart(ap, varg);
    src1 = Ch_VaArg(ap, const CvArr*);
    src2 = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    cvMax(src1, src2, dst);
    Ch_VaEnd(ap);
}

EXPORTCH void cvMinS_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    double scalar;
    CvArr* dst;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    scalar = Ch_VaArg(ap, double);
    dst = Ch_VaArg(ap, CvArr*);
    cvMinS(src, scalar, dst);
    Ch_VaEnd(ap);
}

EXPORTCH void cvMaxS_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    double scalar;
    CvArr* dst;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    scalar = Ch_VaArg(ap, double);
    dst = Ch_VaArg(ap, CvArr*);
    cvMaxS(src, scalar, dst);
    Ch_VaEnd(ap);
}

EXPORTCH void cvCartToPolar_chdl(void *varg) {
    va_list ap;
    const CvArr* x;
    const CvArr* y;
    CvArr* magnitude;
    CvArr* angle;
    int angle_in_degrees;

    Ch_VaStart(ap, varg);
    x = Ch_VaArg(ap, const CvArr*);
    y = Ch_VaArg(ap, const CvArr*);
    magnitude = Ch_VaArg(ap, CvArr*);
    angle = Ch_VaArg(ap, CvArr*);
    angle_in_degrees = Ch_VaArg(ap, int);
    cvCartToPolar(x, y, magnitude, angle, angle_in_degrees);
    Ch_VaEnd(ap);
}

EXPORTCH void cvPolarToCart_chdl(void *varg) {
    va_list ap;
    const CvArr* magnitude;
    const CvArr* angle;
    CvArr* x;
    CvArr* y;
    int angle_in_degrees;

    Ch_VaStart(ap, varg);
    magnitude = Ch_VaArg(ap, const CvArr*);
    angle = Ch_VaArg(ap, const CvArr*);
    x = Ch_VaArg(ap, CvArr*);
    y = Ch_VaArg(ap, CvArr*);
    angle_in_degrees = Ch_VaArg(ap, int);
    cvPolarToCart(magnitude, angle, x, y, angle_in_degrees);
    Ch_VaEnd(ap);
}

EXPORTCH void cvPow_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvArr* dst;
    double power ;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    power  = Ch_VaArg(ap, double);
    cvPow(src, dst, power );
    Ch_VaEnd(ap);
}

EXPORTCH void cvExp_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvArr* dst;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    cvExp(src, dst);
    Ch_VaEnd(ap);
}

EXPORTCH void cvLog_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvArr* dst;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    cvLog(src, dst);
    Ch_VaEnd(ap);
}

EXPORTCH int cvCheckArr_chdl(void *varg) {
    va_list ap;
    const CvArr* arr;
    int flags;
    double minVal;
    double maxVal;
    int retval;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, const CvArr*);
    flags = Ch_VaArg(ap, int);
    minVal = Ch_VaArg(ap, double);
    maxVal = Ch_VaArg(ap, double);
    retval = cvCheckArr(arr, flags, minVal, maxVal);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvRandInit_chdl(void *varg) {
    va_list ap;
    CvRandState* state;
    double param1;
    double param2;
    int seed;
    int disttype;

    Ch_VaStart(ap, varg);
    state = Ch_VaArg(ap, CvRandState*);
    param1 = Ch_VaArg(ap, double);
    param2 = Ch_VaArg(ap, double);
    seed = Ch_VaArg(ap, int);
    disttype = Ch_VaArg(ap, int);
    cvRandInit(state, param1, param2, seed, disttype);
    Ch_VaEnd(ap);
}

EXPORTCH void cvRandSetRange_chdl(void *varg) {
    va_list ap;
    CvRandState* state;
    double param1;
    double param2;
    int index;

    Ch_VaStart(ap, varg);
    state = Ch_VaArg(ap, CvRandState*);
    param1 = Ch_VaArg(ap, double);
    param2 = Ch_VaArg(ap, double);
    index = Ch_VaArg(ap, int);
    cvRandSetRange(state, param1, param2, index);
    Ch_VaEnd(ap);
}

EXPORTCH void cvRand_chdl(void *varg) {
    va_list ap;
    CvRandState* state;
    CvArr* arr ;

    Ch_VaStart(ap, varg);
    state = Ch_VaArg(ap, CvRandState*);
    arr  = Ch_VaArg(ap, CvArr*);
    cvRand(state, arr );
    Ch_VaEnd(ap);
}

EXPORTCH void cvCrossProduct_chdl(void *varg) {
    va_list ap;
    const CvArr* srcA;
    const CvArr* srcB;
    CvArr* dst ;

    Ch_VaStart(ap, varg);
    srcA = Ch_VaArg(ap, const CvArr*);
    srcB = Ch_VaArg(ap, const CvArr*);
    dst  = Ch_VaArg(ap, CvArr*);
    cvCrossProduct(srcA, srcB, dst );
    Ch_VaEnd(ap);
}

EXPORTCH void cvMatMulAdd_chdl(void *varg) {
    va_list ap;
    const CvArr* srcA;
    const CvArr* srcB;
    const CvArr* srcC;
    CvArr* dst;

    Ch_VaStart(ap, varg);
    srcA = Ch_VaArg(ap, const CvArr*);
    srcB = Ch_VaArg(ap, const CvArr*);
    srcC = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    cvMatMulAdd(srcA, srcB, srcC, dst);
    Ch_VaEnd(ap);
}

EXPORTCH void cvGEMM_chdl(void *varg) {
    va_list ap;
    const CvArr* srcA;
    const CvArr* srcB;
    double alpha;
    const CvArr* srcC;
    double beta;
    CvArr* dst;
    int tABC;

    Ch_VaStart(ap, varg);
    srcA = Ch_VaArg(ap, const CvArr*);
    srcB = Ch_VaArg(ap, const CvArr*);
    alpha = Ch_VaArg(ap, double);
    srcC = Ch_VaArg(ap, const CvArr*);
    beta = Ch_VaArg(ap, double);
    dst = Ch_VaArg(ap, CvArr*);
    tABC = Ch_VaArg(ap, int);
    cvGEMM(srcA, srcB, alpha, srcC, beta, dst, tABC);
    Ch_VaEnd(ap);
}

EXPORTCH void cvMatMulAddS_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvArr* dst;
    const CvMat* transform;
    const CvMat* shiftvec;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    transform = Ch_VaArg(ap, const CvMat*);
    shiftvec = Ch_VaArg(ap, const CvMat*);
    cvMatMulAddS(src, dst, transform, shiftvec);
    Ch_VaEnd(ap);
}

EXPORTCH void cvMulTransposed_chdl(void *varg) {
    va_list ap;
    const CvArr* srcarr;
    CvArr* dstarr;
    int order ;

    Ch_VaStart(ap, varg);
    srcarr = Ch_VaArg(ap, const CvArr*);
    dstarr = Ch_VaArg(ap, CvArr*);
    order  = Ch_VaArg(ap, int);
    cvMulTransposed(srcarr, dstarr, order );
    Ch_VaEnd(ap);
}

EXPORTCH void cvTranspose_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvArr* dst ;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    dst  = Ch_VaArg(ap, CvArr*);
    cvTranspose(src, dst );
    Ch_VaEnd(ap);
}

EXPORTCH void cvFlip_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvArr* dst;
    int flip_mode;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    flip_mode = Ch_VaArg(ap, int);
    cvFlip(src, dst, flip_mode);
    Ch_VaEnd(ap);
}

EXPORTCH void cvSVD_chdl(void *varg) {
    va_list ap;
    CvArr* A;
    CvArr* W;
    CvArr* U;
    CvArr* V;
    int flags;

    Ch_VaStart(ap, varg);
    A = Ch_VaArg(ap, CvArr*);
    W = Ch_VaArg(ap, CvArr*);
    U = Ch_VaArg(ap, CvArr*);
    V = Ch_VaArg(ap, CvArr*);
    flags = Ch_VaArg(ap, int);
    cvSVD(A, W, U, V, flags);
    Ch_VaEnd(ap);
}

EXPORTCH void cvSVBkSb_chdl(void *varg) {
    va_list ap;
    const CvArr* warr;
    const CvArr* uarr;
    const CvArr* varr;
    const CvArr* barr;
    CvArr* xarr;
    int flags ;

    Ch_VaStart(ap, varg);
    warr = Ch_VaArg(ap, const CvArr*);
    uarr = Ch_VaArg(ap, const CvArr*);
    varr = Ch_VaArg(ap, const CvArr*);
    barr = Ch_VaArg(ap, const CvArr*);
    xarr = Ch_VaArg(ap, CvArr*);
    flags  = Ch_VaArg(ap, int);
    cvSVBkSb(warr, uarr, varr, barr, xarr, flags );
    Ch_VaEnd(ap);
}

EXPORTCH double cvInvert_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvArr* dst;
    int method;
    double retval;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    method = Ch_VaArg(ap, int);
    retval = cvInvert(src, dst, method);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH int cvSolve_chdl(void *varg) {
    va_list ap;
    const CvArr* A;
    const CvArr* b;
    CvArr* x;
    int method;
    int retval;

    Ch_VaStart(ap, varg);
    A = Ch_VaArg(ap, const CvArr*);
    b = Ch_VaArg(ap, const CvArr*);
    x = Ch_VaArg(ap, CvArr*);
    method = Ch_VaArg(ap, int);
    retval = cvSolve(A, b, x, method);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH double cvDet_chdl(void *varg) {
    va_list ap;
    const CvArr* mat;
    double retval;

    Ch_VaStart(ap, varg);
    mat = Ch_VaArg(ap, const CvArr*);
    retval = cvDet(mat);
    Ch_VaEnd(ap);
    return retval;
}

/*
EXPORTCH CvScalar cvTrace_chdl(void *varg) {
    va_list ap;
    const CvArr* mat;
    CvScalar retval;

    Ch_VaStart(ap, varg);
    mat = Ch_VaArg(ap, const CvArr*);
    retval = cvTrace(mat);
    Ch_VaEnd(ap);
    return retval;
}

*/
EXPORTCH void cvEigenVV_chdl(void *varg) {
    va_list ap;
    CvArr* src;
    CvArr* evects;
    CvArr* evals;
    double eps ;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, CvArr*);
    evects = Ch_VaArg(ap, CvArr*);
    evals = Ch_VaArg(ap, CvArr*);
    eps  = Ch_VaArg(ap, double);
    cvEigenVV(src, evects, evals, eps );
    Ch_VaEnd(ap);
}

EXPORTCH void cvSetIdentity_chdl(void *varg) {
    va_list ap;
    CvArr* mat;
    CvScalar value;

    Ch_VaStart(ap, varg);
    mat = Ch_VaArg(ap, CvArr*);
    value = Ch_VaArg(ap, CvScalar);
    cvSetIdentity(mat, value);
    Ch_VaEnd(ap);
}

EXPORTCH void cvPerspectiveTransform_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvArr* dst;
    const CvArr* mat;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    mat = Ch_VaArg(ap, const CvArr*);
    cvPerspectiveTransform(src, dst, mat);
    Ch_VaEnd(ap);
}

EXPORTCH void cvCalcCovarMatrix_chdl(void *varg) {
    va_list ap;
    const CvArr** vects;
    CvArr* covarMatrix;
    CvArr* avg;

    Ch_VaStart(ap, varg);
    vects = Ch_VaArg(ap, const CvArr**);
    covarMatrix = Ch_VaArg(ap, CvArr*);
    avg = Ch_VaArg(ap, CvArr*);
    cvCalcCovarMatrix(vects, covarMatrix, avg);
    Ch_VaEnd(ap);
}

EXPORTCH double cvMahalanobis_chdl(void *varg) {
    va_list ap;
    const CvArr* srcA;
    const CvArr* srcB;
    CvArr* mat;
    double retval;

    Ch_VaStart(ap, varg);
    srcA = Ch_VaArg(ap, const CvArr*);
    srcB = Ch_VaArg(ap, const CvArr*);
    mat = Ch_VaArg(ap, CvArr*);
    retval = cvMahalanobis(srcA, srcB, mat);
    Ch_VaEnd(ap);
    return retval;
}

/*
EXPORTCH CvScalar cvSum_chdl(void *varg) {
    va_list ap;
    const CvArr* arr;
    CvScalar retval;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, const CvArr*);
    retval = cvSum(arr);
    Ch_VaEnd(ap);
    return retval;
}

*/
EXPORTCH int cvCountNonZero_chdl(void *varg) {
    va_list ap;
    const CvArr* arr;
    int retval;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, const CvArr*);
    retval = cvCountNonZero(arr);
    Ch_VaEnd(ap);
    return retval;
}

/*
EXPORTCH CvScalar cvAvg_chdl(void *varg) {
    va_list ap;
    const CvArr* arr;
    const CvArr* mask;
    CvScalar retval;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, const CvArr*);
    mask = Ch_VaArg(ap, const CvArr*);
    retval = cvAvg(arr, mask);
    Ch_VaEnd(ap);
    return retval;
}

*/
EXPORTCH void cvAvgSdv_chdl(void *varg) {
    va_list ap;
    const CvArr* arr;
    CvScalar* mean;
    CvScalar* std_dev;
    const CvArr* mask;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, const CvArr*);
    mean = Ch_VaArg(ap, CvScalar*);
    std_dev = Ch_VaArg(ap, CvScalar*);
    mask = Ch_VaArg(ap, const CvArr*);
    cvAvgSdv(arr, mean, std_dev, mask);
    Ch_VaEnd(ap);
}

EXPORTCH void cvMinMaxLoc_chdl(void *varg) {
    va_list ap;
    const CvArr* arr;
    double* min_val;
    double* max_val;
    CvPoint* min_loc;
    CvPoint* max_loc;
    const CvArr* mask;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, const CvArr*);
    min_val = Ch_VaArg(ap, double*);
    max_val = Ch_VaArg(ap, double*);
    min_loc = Ch_VaArg(ap, CvPoint*);
    max_loc = Ch_VaArg(ap, CvPoint*);
    mask = Ch_VaArg(ap, const CvArr*);
    cvMinMaxLoc(arr, min_val, max_val, min_loc, max_loc, mask);
    Ch_VaEnd(ap);
}

EXPORTCH double cvNorm_chdl(void *varg) {
    va_list ap;
    const CvArr* arrA;
    const CvArr* arrB;
    int normType;
    const CvArr* mask;
    double retval;

    Ch_VaStart(ap, varg);
    arrA = Ch_VaArg(ap, const CvArr*);
    arrB = Ch_VaArg(ap, const CvArr*);
    normType = Ch_VaArg(ap, int);
    mask = Ch_VaArg(ap, const CvArr*);
    retval = cvNorm(arrA, arrB, normType, mask);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvDFT_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvArr* dst;
    int flags ;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    flags  = Ch_VaArg(ap, int);
    cvDFT(src, dst, flags );
    Ch_VaEnd(ap);
}

EXPORTCH void cvMulCcs_chdl(void *varg) {
    va_list ap;
    const CvArr* srcA;
    const CvArr* srcB;
    CvArr* dst;

    Ch_VaStart(ap, varg);
    srcA = Ch_VaArg(ap, const CvArr*);
    srcB = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    cvMulCcs(srcA, srcB, dst);
    Ch_VaEnd(ap);
}

EXPORTCH void cvDCT_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvArr* dst;
    int flags ;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    flags  = Ch_VaArg(ap, int);
    cvDCT(src, dst, flags );
    Ch_VaEnd(ap);
}

EXPORTCH CvMemStorage* cvCreateMemStorage_chdl(void *varg) {
    va_list ap;
    int block_size;
    CvMemStorage* retval;

    Ch_VaStart(ap, varg);
    block_size = Ch_VaArg(ap, int);
    retval = cvCreateMemStorage(block_size);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CvMemStorage* cvCreateChildMemStorage_chdl(void *varg) {
    va_list ap;
    CvMemStorage* parent ;
    CvMemStorage* retval;

    Ch_VaStart(ap, varg);
    parent  = Ch_VaArg(ap, CvMemStorage*);
    retval = cvCreateChildMemStorage(parent );
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvReleaseMemStorage_chdl(void *varg) {
    va_list ap;
    CvMemStorage** storage;

    Ch_VaStart(ap, varg);
    storage = Ch_VaArg(ap, CvMemStorage**);
    cvReleaseMemStorage(storage);
    Ch_VaEnd(ap);
}

EXPORTCH void cvClearMemStorage_chdl(void *varg) {
    va_list ap;
    CvMemStorage* storage;

    Ch_VaStart(ap, varg);
    storage = Ch_VaArg(ap, CvMemStorage*);
    cvClearMemStorage(storage);
    Ch_VaEnd(ap);
}

EXPORTCH void cvSaveMemStoragePos_chdl(void *varg) {
    va_list ap;
    const CvMemStorage* storage;
    CvMemStoragePos* pos;

    Ch_VaStart(ap, varg);
    storage = Ch_VaArg(ap, const CvMemStorage*);
    pos = Ch_VaArg(ap, CvMemStoragePos*);
    cvSaveMemStoragePos(storage, pos);
    Ch_VaEnd(ap);
}

EXPORTCH void cvRestoreMemStoragePos_chdl(void *varg) {
    va_list ap;
    CvMemStorage* storage;
    CvMemStoragePos* pos;

    Ch_VaStart(ap, varg);
    storage = Ch_VaArg(ap, CvMemStorage*);
    pos = Ch_VaArg(ap, CvMemStoragePos*);
    cvRestoreMemStoragePos(storage, pos);
    Ch_VaEnd(ap);
}

EXPORTCH void* cvMemStorageAlloc_chdl(void *varg) {
    va_list ap;
    CvMemStorage* storage;
    int size;
    void* retval;

    Ch_VaStart(ap, varg);
    storage = Ch_VaArg(ap, CvMemStorage*);
    size = Ch_VaArg(ap, int);
    retval = cvMemStorageAlloc(storage, size);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CvSeq* cvCreateSeq_chdl(void *varg) {
    va_list ap;
    int seq_flags;
    int header_size;
    int elem_size;
    CvMemStorage* storage;
    CvSeq* retval;

    Ch_VaStart(ap, varg);
    seq_flags = Ch_VaArg(ap, int);
    header_size = Ch_VaArg(ap, int);
    elem_size = Ch_VaArg(ap, int);
    storage = Ch_VaArg(ap, CvMemStorage*);
    retval = cvCreateSeq(seq_flags, header_size, elem_size, storage);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvSetSeqBlockSize_chdl(void *varg) {
    va_list ap;
    CvSeq* seq;
    int delta_elements;

    Ch_VaStart(ap, varg);
    seq = Ch_VaArg(ap, CvSeq*);
    delta_elements = Ch_VaArg(ap, int);
    cvSetSeqBlockSize(seq, delta_elements);
    Ch_VaEnd(ap);
}

EXPORTCH char* cvSeqPush_chdl(void *varg) {
    va_list ap;
    CvSeq* seq;
    void* element;
    char* retval;

    Ch_VaStart(ap, varg);
    seq = Ch_VaArg(ap, CvSeq*);
    element = Ch_VaArg(ap, void*);
    retval = cvSeqPush(seq, element);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH char* cvSeqPushFront_chdl(void *varg) {
    va_list ap;
    CvSeq* seq;
    void* element;
    char* retval;

    Ch_VaStart(ap, varg);
    seq = Ch_VaArg(ap, CvSeq*);
    element = Ch_VaArg(ap, void*);
    retval = cvSeqPushFront(seq, element);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvSeqPop_chdl(void *varg) {
    va_list ap;
    CvSeq* seq;
    void* element;

    Ch_VaStart(ap, varg);
    seq = Ch_VaArg(ap, CvSeq*);
    element = Ch_VaArg(ap, void*);
    cvSeqPop(seq, element);
    Ch_VaEnd(ap);
}

EXPORTCH void cvSeqPopFront_chdl(void *varg) {
    va_list ap;
    CvSeq* seq;
    void* element;

    Ch_VaStart(ap, varg);
    seq = Ch_VaArg(ap, CvSeq*);
    element = Ch_VaArg(ap, void*);
    cvSeqPopFront(seq, element);
    Ch_VaEnd(ap);
}

EXPORTCH void cvSeqPushMulti_chdl(void *varg) {
    va_list ap;
    CvSeq* seq;
    void* elements;
    int count;
    int in_front;

    Ch_VaStart(ap, varg);
    seq = Ch_VaArg(ap, CvSeq*);
    elements = Ch_VaArg(ap, void*);
    count = Ch_VaArg(ap, int);
    in_front = Ch_VaArg(ap, int);
    cvSeqPushMulti(seq, elements, count, in_front);
    Ch_VaEnd(ap);
}

EXPORTCH void cvSeqPopMulti_chdl(void *varg) {
    va_list ap;
    CvSeq* seq;
    void* elements;
    int count;
    int in_front;

    Ch_VaStart(ap, varg);
    seq = Ch_VaArg(ap, CvSeq*);
    elements = Ch_VaArg(ap, void*);
    count = Ch_VaArg(ap, int);
    in_front = Ch_VaArg(ap, int);
    cvSeqPopMulti(seq, elements, count, in_front);
    Ch_VaEnd(ap);
}

EXPORTCH char* cvSeqInsert_chdl(void *varg) {
    va_list ap;
    CvSeq* seq;
    int before_index;
    void* element;
    char* retval;

    Ch_VaStart(ap, varg);
    seq = Ch_VaArg(ap, CvSeq*);
    before_index = Ch_VaArg(ap, int);
    element = Ch_VaArg(ap, void*);
    retval = cvSeqInsert(seq, before_index, element);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvSeqRemove_chdl(void *varg) {
    va_list ap;
    CvSeq* seq;
    int index ;

    Ch_VaStart(ap, varg);
    seq = Ch_VaArg(ap, CvSeq*);
    index  = Ch_VaArg(ap, int);
    cvSeqRemove(seq, index );
    Ch_VaEnd(ap);
}

EXPORTCH void cvClearSeq_chdl(void *varg) {
    va_list ap;
    CvSeq* seq;

    Ch_VaStart(ap, varg);
    seq = Ch_VaArg(ap, CvSeq*);
    cvClearSeq(seq);
    Ch_VaEnd(ap);
}

EXPORTCH char* cvGetSeqElem_chdl(void *varg) {
    va_list ap;
    CvSeq* seq;
    int index;
    CvSeqBlock** block;
    char* retval;

    Ch_VaStart(ap, varg);
    seq = Ch_VaArg(ap, CvSeq*);
    index = Ch_VaArg(ap, int);
    block = Ch_VaArg(ap, CvSeqBlock**);
    retval = cvGetSeqElem(seq, index, block);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH int cvSeqElemIdx_chdl(void *varg) {
    va_list ap;
    const CvSeq* seq;
    const void* element;
    CvSeqBlock** block;
    int retval;

    Ch_VaStart(ap, varg);
    seq = Ch_VaArg(ap, const CvSeq*);
    element = Ch_VaArg(ap, const void*);
    block = Ch_VaArg(ap, CvSeqBlock**);
    retval = cvSeqElemIdx(seq, element, block);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvStartAppendToSeq_chdl(void *varg) {
    va_list ap;
    CvSeq* seq;
    CvSeqWriter* writer;

    Ch_VaStart(ap, varg);
    seq = Ch_VaArg(ap, CvSeq*);
    writer = Ch_VaArg(ap, CvSeqWriter*);
    cvStartAppendToSeq(seq, writer);
    Ch_VaEnd(ap);
}

EXPORTCH void cvStartWriteSeq_chdl(void *varg) {
    va_list ap;
    int seq_flags;
    int header_size;
    int elem_size;
    CvMemStorage* storage;
    CvSeqWriter* writer;

    Ch_VaStart(ap, varg);
    seq_flags = Ch_VaArg(ap, int);
    header_size = Ch_VaArg(ap, int);
    elem_size = Ch_VaArg(ap, int);
    storage = Ch_VaArg(ap, CvMemStorage*);
    writer = Ch_VaArg(ap, CvSeqWriter*);
    cvStartWriteSeq(seq_flags, header_size, elem_size, storage, writer);
    Ch_VaEnd(ap);
}

EXPORTCH CvSeq* cvEndWriteSeq_chdl(void *varg) {
    va_list ap;
    CvSeqWriter* writer ;
    CvSeq* retval;

    Ch_VaStart(ap, varg);
    writer  = Ch_VaArg(ap, CvSeqWriter*);
    retval = cvEndWriteSeq(writer );
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvFlushSeqWriter_chdl(void *varg) {
    va_list ap;
    CvSeqWriter* writer ;

    Ch_VaStart(ap, varg);
    writer  = Ch_VaArg(ap, CvSeqWriter*);
    cvFlushSeqWriter(writer );
    Ch_VaEnd(ap);
}

EXPORTCH void cvStartReadSeq_chdl(void *varg) {
    va_list ap;
    const CvSeq* seq;
    CvSeqReader* reader;
    int reverse;

    Ch_VaStart(ap, varg);
    seq = Ch_VaArg(ap, const CvSeq*);
    reader = Ch_VaArg(ap, CvSeqReader*);
    reverse = Ch_VaArg(ap, int);
    cvStartReadSeq(seq, reader, reverse);
    Ch_VaEnd(ap);
}

EXPORTCH int cvGetSeqReaderPos_chdl(void *varg) {
    va_list ap;
    CvSeqReader* reader ;
    int retval;

    Ch_VaStart(ap, varg);
    reader  = Ch_VaArg(ap, CvSeqReader*);
    retval = cvGetSeqReaderPos(reader );
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvSetSeqReaderPos_chdl(void *varg) {
    va_list ap;
    CvSeqReader* reader;
    int index;
    int is_relative;

    Ch_VaStart(ap, varg);
    reader = Ch_VaArg(ap, CvSeqReader*);
    index = Ch_VaArg(ap, int);
    is_relative = Ch_VaArg(ap, int);
    cvSetSeqReaderPos(reader, index, is_relative);
    Ch_VaEnd(ap);
}

EXPORTCH void* cvCvtSeqToArray_chdl(void *varg) {
    va_list ap;
    CvSeq* seq;
    CvArr* arr;
    CvSlice slice;
    void* retval;

    Ch_VaStart(ap, varg);
    seq = Ch_VaArg(ap, CvSeq*);
    arr = Ch_VaArg(ap, CvArr*);
    slice = Ch_VaArg(ap, CvSlice);
    retval = cvCvtSeqToArray(seq, arr, slice);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CvSeq* cvMakeSeqHeaderForArray_chdl(void *varg) {
    va_list ap;
    int seq_type;
    int header_size;
    int elem_size;
    void* elements;
    int total;
    CvSeq* seq;
    CvSeqBlock* block;
    CvSeq* retval;

    Ch_VaStart(ap, varg);
    seq_type = Ch_VaArg(ap, int);
    header_size = Ch_VaArg(ap, int);
    elem_size = Ch_VaArg(ap, int);
    elements = Ch_VaArg(ap, void*);
    total = Ch_VaArg(ap, int);
    seq = Ch_VaArg(ap, CvSeq*);
    block = Ch_VaArg(ap, CvSeqBlock*);
    retval = cvMakeSeqHeaderForArray(seq_type, header_size, elem_size, elements, total, seq, block);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CvSeq* cvSeqSlice_chdl(void *varg) {
    va_list ap;
    CvSeq* seq;
    CvSlice slice;
    CvMemStorage* storage;
    int copy_data;
    CvSeq* retval;

    Ch_VaStart(ap, varg);
    seq = Ch_VaArg(ap, CvSeq*);
    slice = Ch_VaArg(ap, CvSlice);
    storage = Ch_VaArg(ap, CvMemStorage*);
    copy_data = Ch_VaArg(ap, int);
    retval = cvSeqSlice(seq, slice, storage, copy_data);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvSeqRemoveSlice_chdl(void *varg) {
    va_list ap;
    CvSeq* seq;
    CvSlice slice;

    Ch_VaStart(ap, varg);
    seq = Ch_VaArg(ap, CvSeq*);
    slice = Ch_VaArg(ap, CvSlice);
    cvSeqRemoveSlice(seq, slice);
    Ch_VaEnd(ap);
}

EXPORTCH void cvSeqInsertSlice_chdl(void *varg) {
    va_list ap;
    CvSeq* seq;
    int index;
    const CvArr* from_arr;

    Ch_VaStart(ap, varg);
    seq = Ch_VaArg(ap, CvSeq*);
    index = Ch_VaArg(ap, int);
    from_arr = Ch_VaArg(ap, const CvArr*);
    cvSeqInsertSlice(seq, index, from_arr);
    Ch_VaEnd(ap);
}

EXPORTCH void cvSeqSort_chdl(void *varg) {
    va_list ap;
    CvSeq* seq;
    CvCmpFunc func;
    void* userdata;

    Ch_VaStart(ap, varg);
    seq = Ch_VaArg(ap, CvSeq*);
    func = Ch_VaArg(ap, CvCmpFunc);
    userdata = Ch_VaArg(ap, void*);
    cvSeqSort(seq, func, userdata);
    Ch_VaEnd(ap);
}

EXPORTCH void cvSeqInvert_chdl(void *varg) {
    va_list ap;
    CvSeq* seq;

    Ch_VaStart(ap, varg);
    seq = Ch_VaArg(ap, CvSeq*);
    cvSeqInvert(seq);
    Ch_VaEnd(ap);
}

EXPORTCH int cvPartitionSeq_chdl(void *varg) {
    va_list ap;
    CvSeq* seq;
    CvMemStorage* storage;
    CvSeq** comps;
    CvCmpFunc is_equal;
    void* userdata;
    int is_set;
    int retval;

    Ch_VaStart(ap, varg);
    seq = Ch_VaArg(ap, CvSeq*);
    storage = Ch_VaArg(ap, CvMemStorage*);
    comps = Ch_VaArg(ap, CvSeq**);
    is_equal = Ch_VaArg(ap, CvCmpFunc);
    userdata = Ch_VaArg(ap, void*);
    is_set = Ch_VaArg(ap, int);
    retval = cvPartitionSeq(seq, storage, comps, is_equal, userdata, is_set);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvChangeSeqBlock_chdl(void *varg) {
    va_list ap;
    CvSeqReader* reader;
    int direction;

    Ch_VaStart(ap, varg);
    reader = Ch_VaArg(ap, CvSeqReader*);
    direction = Ch_VaArg(ap, int);
    cvChangeSeqBlock(reader, direction);
    Ch_VaEnd(ap);
}

EXPORTCH void cvCreateSeqBlock_chdl(void *varg) {
    va_list ap;
    CvSeqWriter* writer;

    Ch_VaStart(ap, varg);
    writer = Ch_VaArg(ap, CvSeqWriter*);
    cvCreateSeqBlock(writer);
    Ch_VaEnd(ap);
}

EXPORTCH CvSet* cvCreateSet_chdl(void *varg) {
    va_list ap;
    int set_flags;
    int header_size;
    int elem_size;
    CvMemStorage* storage;
    CvSet* retval;

    Ch_VaStart(ap, varg);
    set_flags = Ch_VaArg(ap, int);
    header_size = Ch_VaArg(ap, int);
    elem_size = Ch_VaArg(ap, int);
    storage = Ch_VaArg(ap, CvMemStorage*);
    retval = cvCreateSet(set_flags, header_size, elem_size, storage);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH int cvSetAdd_chdl(void *varg) {
    va_list ap;
    CvSet* set_header;
    CvSetElem* element;
    CvSetElem** inserted_element;
    int retval;

    Ch_VaStart(ap, varg);
    set_header = Ch_VaArg(ap, CvSet*);
    element = Ch_VaArg(ap, CvSetElem*);
    inserted_element = Ch_VaArg(ap, CvSetElem**);
    retval = cvSetAdd(set_header, element, inserted_element);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvSetRemove_chdl(void *varg) {
    va_list ap;
    CvSet* set_header;
    int index;

    Ch_VaStart(ap, varg);
    set_header = Ch_VaArg(ap, CvSet*);
    index = Ch_VaArg(ap, int);
    cvSetRemove(set_header, index);
    Ch_VaEnd(ap);
}

EXPORTCH CvSetElem* cvGetSetElem_chdl(void *varg) {
    va_list ap;
    CvSet* set_header;
    int index;
    CvSetElem* retval;

    Ch_VaStart(ap, varg);
    set_header = Ch_VaArg(ap, CvSet*);
    index = Ch_VaArg(ap, int);
    retval = cvGetSetElem(set_header, index);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvClearSet_chdl(void *varg) {
    va_list ap;
    CvSet* set_header;

    Ch_VaStart(ap, varg);
    set_header = Ch_VaArg(ap, CvSet*);
    cvClearSet(set_header);
    Ch_VaEnd(ap);
}

EXPORTCH CvGraph* cvCreateGraph_chdl(void *varg) {
    va_list ap;
    int graph_flags;
    int header_size;
    int vtx_size;
    int edge_size;
    CvMemStorage* storage;
    CvGraph* retval;

    Ch_VaStart(ap, varg);
    graph_flags = Ch_VaArg(ap, int);
    header_size = Ch_VaArg(ap, int);
    vtx_size = Ch_VaArg(ap, int);
    edge_size = Ch_VaArg(ap, int);
    storage = Ch_VaArg(ap, CvMemStorage*);
    retval = cvCreateGraph(graph_flags, header_size, vtx_size, edge_size, storage);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH int cvGraphAddVtx_chdl(void *varg) {
    va_list ap;
    CvGraph* graph;
    CvGraphVtx* vertex;
    CvGraphVtx** inserted_vertex;
    int retval;

    Ch_VaStart(ap, varg);
    graph = Ch_VaArg(ap, CvGraph*);
    vertex = Ch_VaArg(ap, CvGraphVtx*);
    inserted_vertex = Ch_VaArg(ap, CvGraphVtx**);
    retval = cvGraphAddVtx(graph, vertex, inserted_vertex);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvGraphRemoveVtx_chdl(void *varg) {
    va_list ap;
    CvGraph* graph;
    int index;

    Ch_VaStart(ap, varg);
    graph = Ch_VaArg(ap, CvGraph*);
    index = Ch_VaArg(ap, int);
    cvGraphRemoveVtx(graph, index);
    Ch_VaEnd(ap);
}

EXPORTCH void cvGraphRemoveVtxByPtr_chdl(void *varg) {
    va_list ap;
    CvGraph* graph;
    CvGraphVtx* vtx;

    Ch_VaStart(ap, varg);
    graph = Ch_VaArg(ap, CvGraph*);
    vtx = Ch_VaArg(ap, CvGraphVtx*);
    cvGraphRemoveVtxByPtr(graph, vtx);
    Ch_VaEnd(ap);
}

EXPORTCH int cvGraphAddEdge_chdl(void *varg) {
    va_list ap;
    CvGraph* graph;
    int start_idx;
    int end_idx;
    CvGraphEdge* edge;
    CvGraphEdge** inserted_edge;
    int retval;

    Ch_VaStart(ap, varg);
    graph = Ch_VaArg(ap, CvGraph*);
    start_idx = Ch_VaArg(ap, int);
    end_idx = Ch_VaArg(ap, int);
    edge = Ch_VaArg(ap, CvGraphEdge*);
    inserted_edge = Ch_VaArg(ap, CvGraphEdge**);
    retval = cvGraphAddEdge(graph, start_idx, end_idx, edge, inserted_edge);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH int cvGraphAddEdgeByPtr_chdl(void *varg) {
    va_list ap;
    CvGraph* graph;
    CvGraphVtx* start_vtx;
    CvGraphVtx* end_vtx;
    CvGraphEdge* edge;
    CvGraphEdge** inserted_edge;
    int retval;

    Ch_VaStart(ap, varg);
    graph = Ch_VaArg(ap, CvGraph*);
    start_vtx = Ch_VaArg(ap, CvGraphVtx*);
    end_vtx = Ch_VaArg(ap, CvGraphVtx*);
    edge = Ch_VaArg(ap, CvGraphEdge*);
    inserted_edge = Ch_VaArg(ap, CvGraphEdge**);
    retval = cvGraphAddEdgeByPtr(graph, start_vtx, end_vtx, edge, inserted_edge);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvGraphRemoveEdge_chdl(void *varg) {
    va_list ap;
    CvGraph* graph;
    int start_idx;
    int end_idx;

    Ch_VaStart(ap, varg);
    graph = Ch_VaArg(ap, CvGraph*);
    start_idx = Ch_VaArg(ap, int);
    end_idx = Ch_VaArg(ap, int);
    cvGraphRemoveEdge(graph, start_idx, end_idx);
    Ch_VaEnd(ap);
}

EXPORTCH void cvGraphRemoveEdgeByPtr_chdl(void *varg) {
    va_list ap;
    CvGraph* graph;
    CvGraphVtx* start_vtx;
    CvGraphVtx* end_vtx;

    Ch_VaStart(ap, varg);
    graph = Ch_VaArg(ap, CvGraph*);
    start_vtx = Ch_VaArg(ap, CvGraphVtx*);
    end_vtx = Ch_VaArg(ap, CvGraphVtx*);
    cvGraphRemoveEdgeByPtr(graph, start_vtx, end_vtx);
    Ch_VaEnd(ap);
}

EXPORTCH CvGraphEdge* cvFindGraphEdge_chdl(void *varg) {
    va_list ap;
    CvGraph* graph;
    int start_idx;
    int end_idx;
    CvGraphEdge* retval;

    Ch_VaStart(ap, varg);
    graph = Ch_VaArg(ap, CvGraph*);
    start_idx = Ch_VaArg(ap, int);
    end_idx = Ch_VaArg(ap, int);
    retval = cvFindGraphEdge(graph, start_idx, end_idx);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CvGraphEdge* cvFindGraphEdgeByPtr_chdl(void *varg) {
    va_list ap;
    CvGraph* graph;
    CvGraphVtx* start_vtx;
    CvGraphVtx* end_vtx;
    CvGraphEdge* retval;

    Ch_VaStart(ap, varg);
    graph = Ch_VaArg(ap, CvGraph*);
    start_vtx = Ch_VaArg(ap, CvGraphVtx*);
    end_vtx = Ch_VaArg(ap, CvGraphVtx*);
    retval = cvFindGraphEdgeByPtr(graph, start_vtx, end_vtx);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvClearGraph_chdl(void *varg) {
    va_list ap;
    CvGraph* graph;

    Ch_VaStart(ap, varg);
    graph = Ch_VaArg(ap, CvGraph*);
    cvClearGraph(graph);
    Ch_VaEnd(ap);
}

EXPORTCH int cvGraphVtxDegree_chdl(void *varg) {
    va_list ap;
    CvGraph* graph;
    int vtx_idx;
    int retval;

    Ch_VaStart(ap, varg);
    graph = Ch_VaArg(ap, CvGraph*);
    vtx_idx = Ch_VaArg(ap, int);
    retval = cvGraphVtxDegree(graph, vtx_idx);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH int cvGraphVtxDegreeByPtr_chdl(void *varg) {
    va_list ap;
    CvGraph* graph;
    CvGraphVtx* vtx ;
    int retval;

    Ch_VaStart(ap, varg);
    graph = Ch_VaArg(ap, CvGraph*);
    vtx  = Ch_VaArg(ap, CvGraphVtx*);
    retval = cvGraphVtxDegreeByPtr(graph, vtx );
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvStartScanGraph_chdl(void *varg) {
    va_list ap;
    CvGraph* graph;
    CvGraphScanner* scanner;
    CvGraphVtx* vtx;
    int mask;

    Ch_VaStart(ap, varg);
    graph = Ch_VaArg(ap, CvGraph*);
    scanner = Ch_VaArg(ap, CvGraphScanner*);
    vtx = Ch_VaArg(ap, CvGraphVtx*);
    mask = Ch_VaArg(ap, int);
    cvStartScanGraph(graph, scanner, vtx, mask);
    Ch_VaEnd(ap);
}

EXPORTCH void cvEndScanGraph_chdl(void *varg) {
    va_list ap;
    CvGraphScanner* scanner ;

    Ch_VaStart(ap, varg);
    scanner  = Ch_VaArg(ap, CvGraphScanner*);
    cvEndScanGraph(scanner );
    Ch_VaEnd(ap);
}

EXPORTCH int cvNextGraphItem_chdl(void *varg) {
    va_list ap;
    CvGraphScanner* scanner ;
    int retval;

    Ch_VaStart(ap, varg);
    scanner  = Ch_VaArg(ap, CvGraphScanner*);
    retval = cvNextGraphItem(scanner );
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CvGraph* cvCloneGraph_chdl(void *varg) {
    va_list ap;
    const CvGraph* graph;
    CvMemStorage* storage;
    CvGraph* retval;

    Ch_VaStart(ap, varg);
    graph = Ch_VaArg(ap, const CvGraph*);
    storage = Ch_VaArg(ap, CvMemStorage*);
    retval = cvCloneGraph(graph, storage);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvLUT_chdl(void *varg) {
    va_list ap;
    const CvArr* srcarr;
    CvArr* dstarr;
    const CvArr* lutarr;

    Ch_VaStart(ap, varg);
    srcarr = Ch_VaArg(ap, const CvArr*);
    dstarr = Ch_VaArg(ap, CvArr*);
    lutarr = Ch_VaArg(ap, const CvArr*);
    cvLUT(srcarr, dstarr, lutarr);
    Ch_VaEnd(ap);
}

EXPORTCH void cvSmooth_chdl(void *varg) {
    va_list ap;
    const CvArr* srcarr;
    CvArr* dstarr;
    int smoothtype;
    int param1;
    int param2;

    Ch_VaStart(ap, varg);
    srcarr = Ch_VaArg(ap, const CvArr*);
    dstarr = Ch_VaArg(ap, CvArr*);
    smoothtype = Ch_VaArg(ap, int);
    param1 = Ch_VaArg(ap, int);
    param2 = Ch_VaArg(ap, int);
    cvSmooth(srcarr, dstarr, smoothtype, param1, param2);
    Ch_VaEnd(ap);
}

EXPORTCH void cvIntegral_chdl(void *varg) {
    va_list ap;
    const CvArr* image;
    CvArr* sumImage;
    CvArr* sumSqImage;
    CvArr* tiltedSumImage;

    Ch_VaStart(ap, varg);
    image = Ch_VaArg(ap, const CvArr*);
    sumImage = Ch_VaArg(ap, CvArr*);
    sumSqImage = Ch_VaArg(ap, CvArr*);
    tiltedSumImage = Ch_VaArg(ap, CvArr*);
    cvIntegral(image, sumImage, sumSqImage, tiltedSumImage);
    Ch_VaEnd(ap);
}

EXPORTCH void cvPyrDown_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvArr* dst;
    int filter;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    filter = Ch_VaArg(ap, int);
    cvPyrDown(src, dst, filter);
    Ch_VaEnd(ap);
}

EXPORTCH void cvPyrUp_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvArr* dst;
    int filter;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    filter = Ch_VaArg(ap, int);
    cvPyrUp(src, dst, filter);
    Ch_VaEnd(ap);
}

EXPORTCH void cvPyrSegmentation_chdl(void *varg) {
    va_list ap;
    IplImage* src;
    IplImage* dst;
    CvMemStorage *storage;
    CvSeq ** comp;
    int level;
    double threshold1;
    double threshold2;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, IplImage*);
    dst = Ch_VaArg(ap, IplImage*);
    storage = Ch_VaArg(ap, CvMemStorage *);
    comp = Ch_VaArg(ap, CvSeq ** );
    level = Ch_VaArg(ap, int);
    threshold1 = Ch_VaArg(ap, double);
    threshold2 = Ch_VaArg(ap, double);
    cvPyrSegmentation(src, dst, storage, comp, level, threshold1, threshold2);
    Ch_VaEnd(ap);
}

EXPORTCH void cvSobel_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvArr* dst;
    int xorder;
    int yorder;
    int apertureSize;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    xorder = Ch_VaArg(ap, int);
    yorder = Ch_VaArg(ap, int);
    apertureSize = Ch_VaArg(ap, int);
    cvSobel(src, dst, xorder, yorder, apertureSize);
    Ch_VaEnd(ap);
}

EXPORTCH void cvLaplace_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvArr* dst;
    int apertureSize;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    apertureSize = Ch_VaArg(ap, int);
    cvLaplace(src, dst, apertureSize);
    Ch_VaEnd(ap);
}

EXPORTCH void cvCvtColor_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvArr* dst;
    int colorCvtCode;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    colorCvtCode = Ch_VaArg(ap, int);
    cvCvtColor(src, dst, colorCvtCode);
    Ch_VaEnd(ap);
}

EXPORTCH void cvResize_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvArr* dst;
    int method;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    method = Ch_VaArg(ap, int);
    cvResize(src, dst, method);
    Ch_VaEnd(ap);
}

EXPORTCH IplConvKernel* cvCreateStructuringElementEx_chdl(void *varg) {
    va_list ap;
    int cols;
    int rows;
    int anchorX;
    int anchorY;
    int shape;
    int* values;
    IplConvKernel* retval;

    Ch_VaStart(ap, varg);
    cols = Ch_VaArg(ap, int);
    rows = Ch_VaArg(ap, int);
    anchorX = Ch_VaArg(ap, int);
    anchorY = Ch_VaArg(ap, int);
    shape = Ch_VaArg(ap, int);
    values = Ch_VaArg(ap, int*);
    retval = cvCreateStructuringElementEx(cols, rows, anchorX, anchorY, shape, values);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvReleaseStructuringElement_chdl(void *varg) {
    va_list ap;
    IplConvKernel** element ;

    Ch_VaStart(ap, varg);
    element  = Ch_VaArg(ap, IplConvKernel**);
    cvReleaseStructuringElement(element );
    Ch_VaEnd(ap);
}

EXPORTCH void cvErode_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvArr* dst;
    IplConvKernel* element;
    int iterations;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    element = Ch_VaArg(ap, IplConvKernel*);
    iterations = Ch_VaArg(ap, int);
    cvErode(src, dst, element, iterations);
    Ch_VaEnd(ap);
}

EXPORTCH void cvDilate_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvArr* dst;
    IplConvKernel* element;
    int iterations;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    element = Ch_VaArg(ap, IplConvKernel*);
    iterations = Ch_VaArg(ap, int);
    cvDilate(src, dst, element, iterations);
    Ch_VaEnd(ap);
}

EXPORTCH void cvMorphologyEx_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvArr* dst;
    CvArr* temp;
    IplConvKernel* element;
    int operation;
    int iterations;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    temp = Ch_VaArg(ap, CvArr*);
    element = Ch_VaArg(ap, IplConvKernel*);
    operation = Ch_VaArg(ap, int);
    iterations = Ch_VaArg(ap, int);
    cvMorphologyEx(src, dst, temp, element, operation, iterations);
    Ch_VaEnd(ap);
}

EXPORTCH void cvMoments_chdl(void *varg) {
    va_list ap;
    const CvArr* arr;
    CvMoments* moments;
    int binary;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, const CvArr*);
    moments = Ch_VaArg(ap, CvMoments*);
    binary = Ch_VaArg(ap, int);
    cvMoments(arr, moments, binary);
    Ch_VaEnd(ap);
}

EXPORTCH double cvGetSpatialMoment_chdl(void *varg) {
    va_list ap;
    CvMoments* moments;
    int x_order;
    int y_order;
    double retval;

    Ch_VaStart(ap, varg);
    moments = Ch_VaArg(ap, CvMoments*);
    x_order = Ch_VaArg(ap, int);
    y_order = Ch_VaArg(ap, int);
    retval = cvGetSpatialMoment(moments, x_order, y_order);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH double cvGetCentralMoment_chdl(void *varg) {
    va_list ap;
    CvMoments* moments;
    int x_order;
    int y_order;
    double retval;

    Ch_VaStart(ap, varg);
    moments = Ch_VaArg(ap, CvMoments*);
    x_order = Ch_VaArg(ap, int);
    y_order = Ch_VaArg(ap, int);
    retval = cvGetCentralMoment(moments, x_order, y_order);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH double cvGetNormalizedCentralMoment_chdl(void *varg) {
    va_list ap;
    CvMoments* moments;
    int x_order;
    int y_order;
    double retval;

    Ch_VaStart(ap, varg);
    moments = Ch_VaArg(ap, CvMoments*);
    x_order = Ch_VaArg(ap, int);
    y_order = Ch_VaArg(ap, int);
    retval = cvGetNormalizedCentralMoment(moments, x_order, y_order);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvGetHuMoments_chdl(void *varg) {
    va_list ap;
    CvMoments* moments;
    CvHuMoments* hu_moments;

    Ch_VaStart(ap, varg);
    moments = Ch_VaArg(ap, CvMoments*);
    hu_moments = Ch_VaArg(ap, CvHuMoments*);
    cvGetHuMoments(moments, hu_moments);
    Ch_VaEnd(ap);
}

EXPORTCH void cvLine_chdl(void *varg) {
    va_list ap;
    CvArr* arr;
    CvPoint pt1;
    CvPoint pt2;
    double color;
    int thickness;
    int connectivity;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, CvArr*);
    pt1 = Ch_VaArg(ap, CvPoint);
    pt2 = Ch_VaArg(ap, CvPoint);
    color = Ch_VaArg(ap, double);
    thickness = Ch_VaArg(ap, int);
    connectivity = Ch_VaArg(ap, int);
    cvLine(arr, pt1, pt2, color, thickness, connectivity);
    Ch_VaEnd(ap);
}

EXPORTCH void cvLineAA_chdl(void *varg) {
    va_list ap;
    CvArr* arr;
    CvPoint pt1;
    CvPoint pt2;
    double color;
    int scale;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, CvArr*);
    pt1 = Ch_VaArg(ap, CvPoint);
    pt2 = Ch_VaArg(ap, CvPoint);
    color = Ch_VaArg(ap, double);
    scale = Ch_VaArg(ap, int);
    cvLineAA(arr, pt1, pt2, color, scale);
    Ch_VaEnd(ap);
}

EXPORTCH void cvRectangle_chdl(void *varg) {
    va_list ap;
    CvArr* arr;
    CvPoint pt1;
    CvPoint pt2;
    double color;
    int thickness;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, CvArr*);
    pt1 = Ch_VaArg(ap, CvPoint);
    pt2 = Ch_VaArg(ap, CvPoint);
    color = Ch_VaArg(ap, double);
    thickness = Ch_VaArg(ap, int);
    cvRectangle(arr, pt1, pt2, color, thickness);
    Ch_VaEnd(ap);
}

EXPORTCH void cvCircle_chdl(void *varg) {
    va_list ap;
    CvArr* arr;
    CvPoint center;
    int radius;
    double color;
    int thickness;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, CvArr*);
    center = Ch_VaArg(ap, CvPoint);
    radius = Ch_VaArg(ap, int);
    color = Ch_VaArg(ap, double);
    thickness = Ch_VaArg(ap, int);
    cvCircle(arr, center, radius, color, thickness);
    Ch_VaEnd(ap);
}

EXPORTCH void cvCircleAA_chdl(void *varg) {
    va_list ap;
    CvArr* arr;
    CvPoint center;
    int radius;
    double color;
    int scale;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, CvArr*);
    center = Ch_VaArg(ap, CvPoint);
    radius = Ch_VaArg(ap, int);
    color = Ch_VaArg(ap, double);
    scale = Ch_VaArg(ap, int);
    cvCircleAA(arr, center, radius, color, scale);
    Ch_VaEnd(ap);
}

EXPORTCH void cvEllipse_chdl(void *varg) {
    va_list ap;
    CvArr* arr;
    CvPoint center;
    CvSize axes;
    double angle;
    double startAngle;
    double endAngle;
    double color;
    int thickness;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, CvArr*);
    center = Ch_VaArg(ap, CvPoint);
    axes = Ch_VaArg(ap, CvSize);
    angle = Ch_VaArg(ap, double);
    startAngle = Ch_VaArg(ap, double);
    endAngle = Ch_VaArg(ap, double);
    color = Ch_VaArg(ap, double);
    thickness = Ch_VaArg(ap, int);
    cvEllipse(arr, center, axes, angle, startAngle, endAngle, color, thickness);
    Ch_VaEnd(ap);
}

EXPORTCH void cvEllipseAA_chdl(void *varg) {
    va_list ap;
    CvArr* arr;
    CvPoint center;
    CvSize axes;
    double angle;
    double startAngle;
    double endAngle;
    double color;
    int scale;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, CvArr*);
    center = Ch_VaArg(ap, CvPoint);
    axes = Ch_VaArg(ap, CvSize);
    angle = Ch_VaArg(ap, double);
    startAngle = Ch_VaArg(ap, double);
    endAngle = Ch_VaArg(ap, double);
    color = Ch_VaArg(ap, double);
    scale = Ch_VaArg(ap, int);
    cvEllipseAA(arr, center, axes, angle, startAngle, endAngle, color, scale);
    Ch_VaEnd(ap);
}

EXPORTCH void cvFillConvexPoly_chdl(void *varg) {
    va_list ap;
    CvArr* arr;
    CvPoint* pts;
    int npts;
    double color;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, CvArr*);
    pts = Ch_VaArg(ap, CvPoint*);
    npts = Ch_VaArg(ap, int);
    color = Ch_VaArg(ap, double);
    cvFillConvexPoly(arr, pts, npts, color);
    Ch_VaEnd(ap);
}

EXPORTCH void cvFillPoly_chdl(void *varg) {
    va_list ap;
    CvArr* arr;
    CvPoint** pts;
    int* npts;
    int contours;
    double color ;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, CvArr*);
    pts = Ch_VaArg(ap, CvPoint**);
    npts = Ch_VaArg(ap, int*);
    contours = Ch_VaArg(ap, int);
    color  = Ch_VaArg(ap, double);
    cvFillPoly(arr, pts, npts, contours, color );
    Ch_VaEnd(ap);
}

EXPORTCH void cvPolyLine_chdl(void *varg) {
    va_list ap;
    CvArr* arr;
    CvPoint** pts;
    int* npts;
    int contours;
    int closed;
    double color;
    int thickness;
    int connectivity;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, CvArr*);
    pts = Ch_VaArg(ap, CvPoint**);
    npts = Ch_VaArg(ap, int*);
    contours = Ch_VaArg(ap, int);
    closed = Ch_VaArg(ap, int);
    color = Ch_VaArg(ap, double);
    thickness = Ch_VaArg(ap, int);
    connectivity = Ch_VaArg(ap, int);
    cvPolyLine(arr, pts, npts, contours, closed, color, thickness, connectivity);
    Ch_VaEnd(ap);
}

EXPORTCH void cvPolyLineAA_chdl(void *varg) {
    va_list ap;
    CvArr* arr;
    CvPoint** pts;
    int* npts;
    int contours;
    int closed;
    double color;
    int scale;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, CvArr*);
    pts = Ch_VaArg(ap, CvPoint**);
    npts = Ch_VaArg(ap, int*);
    contours = Ch_VaArg(ap, int);
    closed = Ch_VaArg(ap, int);
    color = Ch_VaArg(ap, double);
    scale = Ch_VaArg(ap, int);
    cvPolyLineAA(arr, pts, npts, contours, closed, color, scale);
    Ch_VaEnd(ap);
}

EXPORTCH void cvInitFont_chdl(void *varg) {
    va_list ap;
    CvFont* font;
    int font_face;
    double hscale;
    double vscale;
    double italic_scale;
    int thickness;

    Ch_VaStart(ap, varg);
    font = Ch_VaArg(ap, CvFont*);
    font_face = Ch_VaArg(ap, int);
    hscale = Ch_VaArg(ap, double);
    vscale = Ch_VaArg(ap, double);
    italic_scale = Ch_VaArg(ap, double);
    thickness = Ch_VaArg(ap, int);
    cvInitFont(font, font_face, hscale, vscale, italic_scale, thickness);
    Ch_VaEnd(ap);
}

EXPORTCH void cvPutText_chdl(void *varg) {
    va_list ap;
    CvArr* arr;
    const char* text;
    CvPoint org;
    CvFont* font;
    double color ;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, CvArr*);
    text = Ch_VaArg(ap, const char*);
    org = Ch_VaArg(ap, CvPoint);
    font = Ch_VaArg(ap, CvFont*);
    color  = Ch_VaArg(ap, double);
    cvPutText(arr, text, org, font, color );
    Ch_VaEnd(ap);
}

EXPORTCH void cvGetTextSize_chdl(void *varg) {
    va_list ap;
    const char* text_string;
    CvFont* font;
    CvSize* text_size;
    int* ymin;

    Ch_VaStart(ap, varg);
    text_string = Ch_VaArg(ap, const char*);
    font = Ch_VaArg(ap, CvFont*);
    text_size = Ch_VaArg(ap, CvSize*);
    ymin = Ch_VaArg(ap, int*);
    cvGetTextSize(text_string, font, text_size, ymin);
    Ch_VaEnd(ap);
}

EXPORTCH int cvInitLineIterator_chdl(void *varg) {
    va_list ap;
    const CvArr* arr;
    CvPoint pt1;
    CvPoint pt2;
    CvLineIterator* lineIterator;
    int connectivity;
    int retval;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, const CvArr*);
    pt1 = Ch_VaArg(ap, CvPoint);
    pt2 = Ch_VaArg(ap, CvPoint);
    lineIterator = Ch_VaArg(ap, CvLineIterator*);
    connectivity = Ch_VaArg(ap, int);
    retval = cvInitLineIterator(arr, pt1, pt2, lineIterator, connectivity);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH int cvSampleLine_chdl(void *varg) {
    va_list ap;
    const CvArr* arr;
    CvPoint pt1;
    CvPoint pt2;
    void* buffer;
    int connectivity;
    int retval;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, const CvArr*);
    pt1 = Ch_VaArg(ap, CvPoint);
    pt2 = Ch_VaArg(ap, CvPoint);
    buffer = Ch_VaArg(ap, void*);
    connectivity = Ch_VaArg(ap, int);
    retval = cvSampleLine(arr, pt1, pt2, buffer, connectivity);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvGetRectSubPix_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvArr* dst;
    CvPoint2D32f center ;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    center  = Ch_VaArg(ap, CvPoint2D32f);
    cvGetRectSubPix(src, dst, center );
    Ch_VaEnd(ap);
}

EXPORTCH void cvGetQuadrangleSubPix_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvArr* dstarr;
    const CvArr* matrixarr;
    int fillOutliers;
    CvScalar fillvalue;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    dstarr = Ch_VaArg(ap, CvArr*);
    matrixarr = Ch_VaArg(ap, const CvArr*);
    fillOutliers = Ch_VaArg(ap, int);
    fillvalue = Ch_VaArg(ap, CvScalar);
    cvGetQuadrangleSubPix(src, dstarr, matrixarr, fillOutliers, fillvalue);
    Ch_VaEnd(ap);
}

EXPORTCH void cvMatchTemplate_chdl(void *varg) {
    va_list ap;
    const CvArr* arr;
    const CvArr* templ;
    CvArr* result;
    int method ;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, const CvArr*);
    templ = Ch_VaArg(ap, const CvArr*);
    result = Ch_VaArg(ap, CvArr*);
    method  = Ch_VaArg(ap, int);
    cvMatchTemplate(arr, templ, result, method );
    Ch_VaEnd(ap);
}

EXPORTCH float cvCalcEMD2_chdl(void *varg) {
    va_list ap;
    const CvArr* signature1;
    const CvArr* signature2;
    CvDisType dist_type;
    CvDistanceFunction dist_func;
    const CvArr* cost_matrix;
    CvArr* flow;
    float* lower_bound;
    void* user_param;
    float retval;

    Ch_VaStart(ap, varg);
    signature1 = Ch_VaArg(ap, const CvArr*);
    signature2 = Ch_VaArg(ap, const CvArr*);
    dist_type = Ch_VaArg(ap, CvDisType);
    dist_func = Ch_VaArg(ap, CvDistanceFunction);
    cost_matrix = Ch_VaArg(ap, const CvArr*);
    flow = Ch_VaArg(ap, CvArr*);
    lower_bound = Ch_VaArg(ap, float*);
    user_param = Ch_VaArg(ap, void*);
    retval = cvCalcEMD2(signature1, signature2, dist_type, dist_func, cost_matrix, flow, lower_bound, user_param);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH int cvFindContours_chdl(void *varg) {
    va_list ap;
    CvArr* arr;
    CvMemStorage* storage;
    CvSeq** firstContour;
    int headerSize;
    CvContourRetrievalMode mode;
    CvChainApproxMethod method;
    int retval;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, CvArr*);
    storage = Ch_VaArg(ap, CvMemStorage*);
    firstContour = Ch_VaArg(ap, CvSeq**);
    headerSize = Ch_VaArg(ap, int);
    mode = Ch_VaArg(ap, CvContourRetrievalMode);
    method = Ch_VaArg(ap, CvChainApproxMethod);
    retval = cvFindContours(arr, storage, firstContour, headerSize, mode, method);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CvContourScanner cvStartFindContours_chdl(void *varg) {
    va_list ap;
    CvArr* arr;
    CvMemStorage* storage;
    int header_size;
    CvContourRetrievalMode mode;
    CvChainApproxMethod method;
    CvContourScanner retval;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, CvArr*);
    storage = Ch_VaArg(ap, CvMemStorage*);
    header_size = Ch_VaArg(ap, int);
    mode = Ch_VaArg(ap, CvContourRetrievalMode);
    method = Ch_VaArg(ap, CvChainApproxMethod);
    retval = cvStartFindContours(arr, storage, header_size, mode, method);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CvSeq* cvFindNextContour_chdl(void *varg) {
    va_list ap;
    CvContourScanner scanner ;
    CvSeq* retval;

    Ch_VaStart(ap, varg);
    scanner  = Ch_VaArg(ap, CvContourScanner);
    retval = cvFindNextContour(scanner );
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvSubstituteContour_chdl(void *varg) {
    va_list ap;
    CvContourScanner scanner;
    CvSeq* newContour;

    Ch_VaStart(ap, varg);
    scanner = Ch_VaArg(ap, CvContourScanner);
    newContour = Ch_VaArg(ap, CvSeq*);
    cvSubstituteContour(scanner, newContour);
    Ch_VaEnd(ap);
}

EXPORTCH CvSeq* cvEndFindContours_chdl(void *varg) {
    va_list ap;
    CvContourScanner* scanner ;
    CvSeq* retval;

    Ch_VaStart(ap, varg);
    scanner  = Ch_VaArg(ap, CvContourScanner*);
    retval = cvEndFindContours(scanner );
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvDrawContours_chdl(void *varg) {
    va_list ap;
    CvArr *img;
    CvSeq* contour;
    double external_color;
    double hole_color;
    int max_level;
    int thickness;
    int connectivity;

    Ch_VaStart(ap, varg);
    img = Ch_VaArg(ap, CvArr *);
    contour = Ch_VaArg(ap, CvSeq*);
    external_color = Ch_VaArg(ap, double);
    hole_color = Ch_VaArg(ap, double);
    max_level = Ch_VaArg(ap, int);
    thickness = Ch_VaArg(ap, int);
    connectivity = Ch_VaArg(ap, int);
    cvDrawContours(img, contour, external_color, hole_color, max_level, thickness, connectivity);
    Ch_VaEnd(ap);
}

EXPORTCH void cvInitTreeNodeIterator_chdl(void *varg) {
    va_list ap;
    CvTreeNodeIterator* treeIterator;
    const void* first;
    int maxLevel ;

    Ch_VaStart(ap, varg);
    treeIterator = Ch_VaArg(ap, CvTreeNodeIterator*);
    first = Ch_VaArg(ap, const void*);
    maxLevel  = Ch_VaArg(ap, int);
    cvInitTreeNodeIterator(treeIterator, first, maxLevel );
    Ch_VaEnd(ap);
}

EXPORTCH void* cvNextTreeNode_chdl(void *varg) {
    va_list ap;
    CvTreeNodeIterator* treeIterator;
    void* retval;

    Ch_VaStart(ap, varg);
    treeIterator = Ch_VaArg(ap, CvTreeNodeIterator*);
    retval = cvNextTreeNode(treeIterator);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void* cvPrevTreeNode_chdl(void *varg) {
    va_list ap;
    CvTreeNodeIterator* treeIterator;
    void* retval;

    Ch_VaStart(ap, varg);
    treeIterator = Ch_VaArg(ap, CvTreeNodeIterator*);
    retval = cvPrevTreeNode(treeIterator);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvInsertNodeIntoTree_chdl(void *varg) {
    va_list ap;
    void* node;
    void* parent;
    void* frame ;

    Ch_VaStart(ap, varg);
    node = Ch_VaArg(ap, void*);
    parent = Ch_VaArg(ap, void*);
    frame  = Ch_VaArg(ap, void*);
    cvInsertNodeIntoTree(node, parent, frame );
    Ch_VaEnd(ap);
}

EXPORTCH void cvRemoveNodeFromTree_chdl(void *varg) {
    va_list ap;
    void* node;
    void* frame;

    Ch_VaStart(ap, varg);
    node = Ch_VaArg(ap, void*);
    frame = Ch_VaArg(ap, void*);
    cvRemoveNodeFromTree(node, frame);
    Ch_VaEnd(ap);
}

EXPORTCH CvSeq* cvTreeToNodeSeq_chdl(void *varg) {
    va_list ap;
    const void* first;
    int header_size;
    CvMemStorage* storage;
    CvSeq* retval;

    Ch_VaStart(ap, varg);
    first = Ch_VaArg(ap, const void*);
    header_size = Ch_VaArg(ap, int);
    storage = Ch_VaArg(ap, CvMemStorage*);
    retval = cvTreeToNodeSeq(first, header_size, storage);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CvSeq* cvApproxChains_chdl(void *varg) {
    va_list ap;
    CvSeq* src_seq;
    CvMemStorage* storage;
    CvChainApproxMethod method;
    double parameter;
    int minimal_perimeter;
    int recursive;
    CvSeq* retval;

    Ch_VaStart(ap, varg);
    src_seq = Ch_VaArg(ap, CvSeq*);
    storage = Ch_VaArg(ap, CvMemStorage*);
    method = Ch_VaArg(ap, CvChainApproxMethod);
    parameter = Ch_VaArg(ap, double);
    minimal_perimeter = Ch_VaArg(ap, int);
    recursive = Ch_VaArg(ap, int);
    retval = cvApproxChains(src_seq, storage, method, parameter, minimal_perimeter, recursive);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvStartReadChainPoints_chdl(void *varg) {
    va_list ap;
    CvChain* chain;
    CvChainPtReader* reader ;

    Ch_VaStart(ap, varg);
    chain = Ch_VaArg(ap, CvChain*);
    reader  = Ch_VaArg(ap, CvChainPtReader*);
    cvStartReadChainPoints(chain, reader );
    Ch_VaEnd(ap);
}

EXPORTCH CvPoint cvReadChainPoint_chdl(void *varg) {
    va_list ap;
    CvChainPtReader* reader ;
    CvPoint retval;

    Ch_VaStart(ap, varg);
    reader  = Ch_VaArg(ap, CvChainPtReader*);
    retval = cvReadChainPoint(reader );
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvAbsDiff_chdl(void *varg) {
    va_list ap;
    const CvArr* srcA;
    const CvArr* srcB;
    CvArr* dst ;

    Ch_VaStart(ap, varg);
    srcA = Ch_VaArg(ap, const CvArr*);
    srcB = Ch_VaArg(ap, const CvArr*);
    dst  = Ch_VaArg(ap, CvArr*);
    cvAbsDiff(srcA, srcB, dst );
    Ch_VaEnd(ap);
}

EXPORTCH void cvAbsDiffS_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvArr* dst;
    CvScalar value;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    value = Ch_VaArg(ap, CvScalar);
    cvAbsDiffS(src, dst, value);
    Ch_VaEnd(ap);
}

EXPORTCH void cvCalcOpticalFlowLK_chdl(void *varg) {
    va_list ap;
    const CvArr* srcA;
    const CvArr* srcB;
    CvSize winSize;
    CvArr* velx;
    CvArr* vely ;

    Ch_VaStart(ap, varg);
    srcA = Ch_VaArg(ap, const CvArr*);
    srcB = Ch_VaArg(ap, const CvArr*);
    winSize = Ch_VaArg(ap, CvSize);
    velx = Ch_VaArg(ap, CvArr*);
    vely  = Ch_VaArg(ap, CvArr*);
    cvCalcOpticalFlowLK(srcA, srcB, winSize, velx, vely );
    Ch_VaEnd(ap);
}

EXPORTCH void cvCalcOpticalFlowBM_chdl(void *varg) {
    va_list ap;
    const CvArr* srcA;
    const CvArr* srcB;
    CvSize blockSize;
    CvSize shiftSize;
    CvSize maxRange;
    int usePrevious;
    CvArr* velx;
    CvArr* vely ;

    Ch_VaStart(ap, varg);
    srcA = Ch_VaArg(ap, const CvArr*);
    srcB = Ch_VaArg(ap, const CvArr*);
    blockSize = Ch_VaArg(ap, CvSize);
    shiftSize = Ch_VaArg(ap, CvSize);
    maxRange = Ch_VaArg(ap, CvSize);
    usePrevious = Ch_VaArg(ap, int);
    velx = Ch_VaArg(ap, CvArr*);
    vely  = Ch_VaArg(ap, CvArr*);
    cvCalcOpticalFlowBM(srcA, srcB, blockSize, shiftSize, maxRange, usePrevious, velx, vely );
    Ch_VaEnd(ap);
}

EXPORTCH void cvCalcOpticalFlowHS_chdl(void *varg) {
    va_list ap;
    const CvArr* srcA;
    const CvArr* srcB;
    int usePrevious;
    CvArr* velx;
    CvArr* vely;
    double lambda;
    CvTermCriteria criteria ;

    Ch_VaStart(ap, varg);
    srcA = Ch_VaArg(ap, const CvArr*);
    srcB = Ch_VaArg(ap, const CvArr*);
    usePrevious = Ch_VaArg(ap, int);
    velx = Ch_VaArg(ap, CvArr*);
    vely = Ch_VaArg(ap, CvArr*);
    lambda = Ch_VaArg(ap, double);
    criteria  = Ch_VaArg(ap, CvTermCriteria);
    cvCalcOpticalFlowHS(srcA, srcB, usePrevious, velx, vely, lambda, criteria );
    Ch_VaEnd(ap);
}

EXPORTCH void cvCalcOpticalFlowPyrLK_chdl(void *varg) {
    va_list ap;
    const CvArr* imgA;
    const CvArr* imgB;
    CvArr* pyrA;
    CvArr* pyrB;
    CvPoint2D32f* featuresA;
    CvPoint2D32f* featuresB;
    int count;
    CvSize winSize;
    int level;
    char* status;
    float* error;
    CvTermCriteria criteria;
    int flags;

    Ch_VaStart(ap, varg);
    imgA = Ch_VaArg(ap, const CvArr*);
    imgB = Ch_VaArg(ap, const CvArr*);
    pyrA = Ch_VaArg(ap, CvArr*);
    pyrB = Ch_VaArg(ap, CvArr*);
    featuresA = Ch_VaArg(ap, CvPoint2D32f*);
    featuresB = Ch_VaArg(ap, CvPoint2D32f*);
    count = Ch_VaArg(ap, int);
    winSize = Ch_VaArg(ap, CvSize);
    level = Ch_VaArg(ap, int);
    status = Ch_VaArg(ap, char*);
    error = Ch_VaArg(ap, float*);
    criteria = Ch_VaArg(ap, CvTermCriteria);
    flags = Ch_VaArg(ap, int);
    cvCalcOpticalFlowPyrLK(imgA, imgB, pyrA, pyrB, featuresA, featuresB, count, winSize, level, status, error, criteria, flags);
    Ch_VaEnd(ap);
}

EXPORTCH void cvCalcAffineFlowPyrLK_chdl(void *varg) {
    va_list ap;
    const CvArr* imgA;
    const CvArr* imgB;
    CvArr* pyrA;
    CvArr* pyrB;
    CvPoint2D32f* featuresA;
    CvPoint2D32f* featuresB;
    float* matrices;
    int count;
    CvSize winSize;
    int level;
    char* status;
    float* error;
    CvTermCriteria criteria;
    int flags;

    Ch_VaStart(ap, varg);
    imgA = Ch_VaArg(ap, const CvArr*);
    imgB = Ch_VaArg(ap, const CvArr*);
    pyrA = Ch_VaArg(ap, CvArr*);
    pyrB = Ch_VaArg(ap, CvArr*);
    featuresA = Ch_VaArg(ap, CvPoint2D32f*);
    featuresB = Ch_VaArg(ap, CvPoint2D32f*);
    matrices = Ch_VaArg(ap, float*);
    count = Ch_VaArg(ap, int);
    winSize = Ch_VaArg(ap, CvSize);
    level = Ch_VaArg(ap, int);
    status = Ch_VaArg(ap, char*);
    error = Ch_VaArg(ap, float*);
    criteria = Ch_VaArg(ap, CvTermCriteria);
    flags = Ch_VaArg(ap, int);
    cvCalcAffineFlowPyrLK(imgA, imgB, pyrA, pyrB, featuresA, featuresB, matrices, count, winSize, level, status, error, criteria, flags);
    Ch_VaEnd(ap);
}

EXPORTCH void cvUpdateMotionHistory_chdl(void *varg) {
    va_list ap;
    const CvArr* silhouette;
    CvArr* mhi;
    double timestamp;
    double mhiDuration;

    Ch_VaStart(ap, varg);
    silhouette = Ch_VaArg(ap, const CvArr*);
    mhi = Ch_VaArg(ap, CvArr*);
    timestamp = Ch_VaArg(ap, double);
    mhiDuration = Ch_VaArg(ap, double);
    cvUpdateMotionHistory(silhouette, mhi, timestamp, mhiDuration);
    Ch_VaEnd(ap);
}

EXPORTCH void cvCalcMotionGradient_chdl(void *varg) {
    va_list ap;
    const CvArr* mhi;
    CvArr* mask;
    CvArr* orientation;
    double delta1;
    double delta2;
    int aperture_size;

    Ch_VaStart(ap, varg);
    mhi = Ch_VaArg(ap, const CvArr*);
    mask = Ch_VaArg(ap, CvArr*);
    orientation = Ch_VaArg(ap, CvArr*);
    delta1 = Ch_VaArg(ap, double);
    delta2 = Ch_VaArg(ap, double);
    aperture_size = Ch_VaArg(ap, int);
    cvCalcMotionGradient(mhi, mask, orientation, delta1, delta2, aperture_size);
    Ch_VaEnd(ap);
}

EXPORTCH double cvCalcGlobalOrientation_chdl(void *varg) {
    va_list ap;
    const CvArr* orientation;
    const CvArr* mask;
    const CvArr* mhi;
    double curr_mhi_timestamp;
    double mhi_duration ;
    double retval;

    Ch_VaStart(ap, varg);
    orientation = Ch_VaArg(ap, const CvArr*);
    mask = Ch_VaArg(ap, const CvArr*);
    mhi = Ch_VaArg(ap, const CvArr*);
    curr_mhi_timestamp = Ch_VaArg(ap, double);
    mhi_duration  = Ch_VaArg(ap, double);
    retval = cvCalcGlobalOrientation(orientation, mask, mhi, curr_mhi_timestamp, mhi_duration );
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CvSeq* cvSegmentMotion_chdl(void *varg) {
    va_list ap;
    const CvArr* mhi;
    CvArr* seg_mask;
    CvMemStorage* storage;
    double timestamp;
    double seg_thresh;
    CvSeq* retval;

    Ch_VaStart(ap, varg);
    mhi = Ch_VaArg(ap, const CvArr*);
    seg_mask = Ch_VaArg(ap, CvArr*);
    storage = Ch_VaArg(ap, CvMemStorage*);
    timestamp = Ch_VaArg(ap, double);
    seg_thresh = Ch_VaArg(ap, double);
    retval = cvSegmentMotion(mhi, seg_mask, storage, timestamp, seg_thresh);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvAcc_chdl(void *varg) {
    va_list ap;
    const CvArr* image;
    CvArr* sum;
    const CvArr* mask;

    Ch_VaStart(ap, varg);
    image = Ch_VaArg(ap, const CvArr*);
    sum = Ch_VaArg(ap, CvArr*);
    mask = Ch_VaArg(ap, const CvArr*);
    cvAcc(image, sum, mask);
    Ch_VaEnd(ap);
}

EXPORTCH void cvSquareAcc_chdl(void *varg) {
    va_list ap;
    const CvArr* image;
    CvArr* sqSum;
    const CvArr* mask;

    Ch_VaStart(ap, varg);
    image = Ch_VaArg(ap, const CvArr*);
    sqSum = Ch_VaArg(ap, CvArr*);
    mask = Ch_VaArg(ap, const CvArr*);
    cvSquareAcc(image, sqSum, mask);
    Ch_VaEnd(ap);
}

EXPORTCH void cvMultiplyAcc_chdl(void *varg) {
    va_list ap;
    const CvArr* imgA;
    const CvArr* imgB;
    CvArr* acc;
    const CvArr* mask;

    Ch_VaStart(ap, varg);
    imgA = Ch_VaArg(ap, const CvArr*);
    imgB = Ch_VaArg(ap, const CvArr*);
    acc = Ch_VaArg(ap, CvArr*);
    mask = Ch_VaArg(ap, const CvArr*);
    cvMultiplyAcc(imgA, imgB, acc, mask);
    Ch_VaEnd(ap);
}

EXPORTCH void cvRunningAvg_chdl(void *varg) {
    va_list ap;
    const CvArr* imgY;
    CvArr* imgU;
    double alpha;
    const CvArr* mask;

    Ch_VaStart(ap, varg);
    imgY = Ch_VaArg(ap, const CvArr*);
    imgU = Ch_VaArg(ap, CvArr*);
    alpha = Ch_VaArg(ap, double);
    mask = Ch_VaArg(ap, const CvArr*);
    cvRunningAvg(imgY, imgU, alpha, mask);
    Ch_VaEnd(ap);
}

EXPORTCH int cvCamShift_chdl(void *varg) {
    va_list ap;
    const CvArr* imgProb;
    CvRect windowIn;
    CvTermCriteria criteria;
    CvConnectedComp* out;
    CvBox2D* box ;
    int retval;

    Ch_VaStart(ap, varg);
    imgProb = Ch_VaArg(ap, const CvArr*);
    windowIn = Ch_VaArg(ap, CvRect);
    criteria = Ch_VaArg(ap, CvTermCriteria);
    out = Ch_VaArg(ap, CvConnectedComp*);
    box  = Ch_VaArg(ap, CvBox2D*);
    retval = cvCamShift(imgProb, windowIn, criteria, out, box );
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH int cvMeanShift_chdl(void *varg) {
    va_list ap;
    const CvArr* imgProb;
    CvRect windowIn;
    CvTermCriteria criteria;
    CvConnectedComp* out;
    int retval;

    Ch_VaStart(ap, varg);
    imgProb = Ch_VaArg(ap, const CvArr*);
    windowIn = Ch_VaArg(ap, CvRect);
    criteria = Ch_VaArg(ap, CvTermCriteria);
    out = Ch_VaArg(ap, CvConnectedComp*);
    retval = cvMeanShift(imgProb, windowIn, criteria, out);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CvConDensation* cvCreateConDensation_chdl(void *varg) {
    va_list ap;
    int DP;
    int MP;
    int SamplesNum;
    CvConDensation* retval;

    Ch_VaStart(ap, varg);
    DP = Ch_VaArg(ap, int);
    MP = Ch_VaArg(ap, int);
    SamplesNum = Ch_VaArg(ap, int);
    retval = cvCreateConDensation(DP, MP, SamplesNum);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvReleaseConDensation_chdl(void *varg) {
    va_list ap;
    CvConDensation** ConDensation;

    Ch_VaStart(ap, varg);
    ConDensation = Ch_VaArg(ap, CvConDensation**);
    cvReleaseConDensation(ConDensation);
    Ch_VaEnd(ap);
}

EXPORTCH void cvConDensUpdateByTime_chdl(void *varg) {
    va_list ap;
    CvConDensation* ConDens;

    Ch_VaStart(ap, varg);
    ConDens = Ch_VaArg(ap, CvConDensation*);
    cvConDensUpdateByTime(ConDens);
    Ch_VaEnd(ap);
}

EXPORTCH void cvConDensInitSampleSet_chdl(void *varg) {
    va_list ap;
    CvConDensation* conDens;
    CvMat* lowerBound;
    CvMat* upperBound;

    Ch_VaStart(ap, varg);
    conDens = Ch_VaArg(ap, CvConDensation*);
    lowerBound = Ch_VaArg(ap, CvMat*);
    upperBound = Ch_VaArg(ap, CvMat* );
    cvConDensInitSampleSet(conDens, lowerBound, upperBound);
    Ch_VaEnd(ap);
}

EXPORTCH CvKalman* cvCreateKalman_chdl(void *varg) {
    va_list ap;
    int dynamParams;
    int measureParams;
    int controlParams;
    CvKalman* retval;

    Ch_VaStart(ap, varg);
    dynamParams = Ch_VaArg(ap, int);
    measureParams = Ch_VaArg(ap, int);
    controlParams = Ch_VaArg(ap, int);
    retval = cvCreateKalman(dynamParams, measureParams, controlParams);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvReleaseKalman_chdl(void *varg) {
    va_list ap;
    CvKalman** Kalman;

    Ch_VaStart(ap, varg);
    Kalman = Ch_VaArg(ap, CvKalman**);
    cvReleaseKalman(Kalman);
    Ch_VaEnd(ap);
}

EXPORTCH const CvMat* cvKalmanPredict_chdl(void *varg) {
    va_list ap;
    CvKalman* Kalman;
    const CvMat* control;
    const CvMat* retval;

    Ch_VaStart(ap, varg);
    Kalman = Ch_VaArg(ap, CvKalman*);
    control = Ch_VaArg(ap, const CvMat*);
    retval = cvKalmanPredict(Kalman, control);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH const CvMat* cvKalmanCorrect_chdl(void *varg) {
    va_list ap;
    CvKalman* Kalman;
    const CvMat* measurement;
    const CvMat* retval;

    Ch_VaStart(ap, varg);
    Kalman = Ch_VaArg(ap, CvKalman*);
    measurement = Ch_VaArg(ap, const CvMat*);
    retval = cvKalmanCorrect(Kalman, measurement);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvInitSubdivDelaunay2D_chdl(void *varg) {
    va_list ap;
    CvSubdiv2D* subdiv;
    CvRect rect ;

    Ch_VaStart(ap, varg);
    subdiv = Ch_VaArg(ap, CvSubdiv2D*);
    rect  = Ch_VaArg(ap, CvRect);
    cvInitSubdivDelaunay2D(subdiv, rect );
    Ch_VaEnd(ap);
}

EXPORTCH CvSubdiv2D* cvCreateSubdiv2D_chdl(void *varg) {
    va_list ap;
    int subdiv_type;
    int header_size;
    int vtx_size;
    int quadedge_size;
    CvMemStorage* storage ;
    CvSubdiv2D* retval;

    Ch_VaStart(ap, varg);
    subdiv_type = Ch_VaArg(ap, int);
    header_size = Ch_VaArg(ap, int);
    vtx_size = Ch_VaArg(ap, int);
    quadedge_size = Ch_VaArg(ap, int);
    storage  = Ch_VaArg(ap, CvMemStorage*);
    retval = cvCreateSubdiv2D(subdiv_type, header_size, vtx_size, quadedge_size, storage );
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CvSubdiv2DPoint* cvSubdivDelaunay2DInsert_chdl(void *varg) {
    va_list ap;
    CvSubdiv2D* subdiv;
    CvPoint2D32f pt;
    CvSubdiv2DPoint* retval;

    Ch_VaStart(ap, varg);
    subdiv = Ch_VaArg(ap, CvSubdiv2D*);
    pt = Ch_VaArg(ap, CvPoint2D32f);
    retval = cvSubdivDelaunay2DInsert(subdiv, pt);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CvSubdiv2DPointLocation cvSubdiv2DLocate_chdl(void *varg) {
    va_list ap;
    CvSubdiv2D* subdiv;
    CvPoint2D32f pt;
    CvSubdiv2DEdge *_edge;
    CvSubdiv2DPoint** _point;
    CvSubdiv2DPointLocation retval;

    Ch_VaStart(ap, varg);
    subdiv = Ch_VaArg(ap, CvSubdiv2D*);
    pt = Ch_VaArg(ap, CvPoint2D32f);
    _edge = Ch_VaArg(ap, CvSubdiv2DEdge *);
    _point = Ch_VaArg(ap, CvSubdiv2DPoint**);
    retval = cvSubdiv2DLocate(subdiv, pt, _edge, _point);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvCalcSubdivVoronoi2D_chdl(void *varg) {
    va_list ap;
    CvSubdiv2D* subdiv;

    Ch_VaStart(ap, varg);
    subdiv = Ch_VaArg(ap, CvSubdiv2D*);
    cvCalcSubdivVoronoi2D(subdiv);
    Ch_VaEnd(ap);
}

EXPORTCH void cvClearSubdivVoronoi2D_chdl(void *varg) {
    va_list ap;
    CvSubdiv2D* subdiv;

    Ch_VaStart(ap, varg);
    subdiv = Ch_VaArg(ap, CvSubdiv2D*);
    cvClearSubdivVoronoi2D(subdiv);
    Ch_VaEnd(ap);
}

EXPORTCH CvSubdiv2DPoint* cvFindNearestPoint2D_chdl(void *varg) {
    va_list ap;
    CvSubdiv2D* subdiv;
    CvPoint2D32f pt;
    CvSubdiv2DPoint* retval;

    Ch_VaStart(ap, varg);
    subdiv = Ch_VaArg(ap, CvSubdiv2D*);
    pt = Ch_VaArg(ap, CvPoint2D32f);
    retval = cvFindNearestPoint2D(subdiv, pt);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CvSeq* cvApproxPoly_chdl(void *varg) {
    va_list ap;
    const void* src_seq;
    int header_size;
    CvMemStorage* storage;
    int method;
    double parameter;
    int parameter2;
    CvSeq* retval;

    Ch_VaStart(ap, varg);
    src_seq = Ch_VaArg(ap, const void*);
    header_size = Ch_VaArg(ap, int);
    storage = Ch_VaArg(ap, CvMemStorage*);
    method = Ch_VaArg(ap, int);
    parameter = Ch_VaArg(ap, double);
    parameter2 = Ch_VaArg(ap, int);
    retval = cvApproxPoly(src_seq, header_size, storage, method, parameter, parameter2);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH double cvArcLength_chdl(void *varg) {
    va_list ap;
    const void* curve;
    CvSlice slice;
    int is_closed;
    double retval;

    Ch_VaStart(ap, varg);
    curve = Ch_VaArg(ap, const void*);
    slice = Ch_VaArg(ap, CvSlice);
    is_closed = Ch_VaArg(ap, int);
    retval = cvArcLength(curve, slice, is_closed);
    Ch_VaEnd(ap);
    return retval;
}

/*
EXPORTCH CvRect cvBoundingRect_chdl(void *varg) {
    va_list ap;
    const void* points;
    int update;
    CvRect retval;

    Ch_VaStart(ap, varg);
    points = Ch_VaArg(ap, const void*);
    update = Ch_VaArg(ap, int);
    retval = cvBoundingRect(points, update);
    Ch_VaEnd(ap);
    return retval;
}

*/
EXPORTCH double cvContourArea_chdl(void *varg) {
    va_list ap;
    const void* contour;
    CvSlice slice;
    double retval;

    Ch_VaStart(ap, varg);
    contour = Ch_VaArg(ap, const void*);
    slice = Ch_VaArg(ap, CvSlice);
    retval = cvContourArea(contour, slice);
    Ch_VaEnd(ap);
    return retval;
}

/*
EXPORTCH CvBox2D cvMinAreaRect2_chdl(void *varg) {
    va_list ap;
    const void* points;
    CvMemStorage* storage;
    CvBox2D retval;

    Ch_VaStart(ap, varg);
    points = Ch_VaArg(ap, const void*);
    storage = Ch_VaArg(ap, CvMemStorage*);
    retval = cvMinAreaRect2(points, storage);
    Ch_VaEnd(ap);
    return retval;
}

*/
EXPORTCH void cvMinEnclosingCircle_chdl(void *varg) {
    va_list ap;
    const void* points;
    CvPoint2D32f* center;
    float* radius ;

    Ch_VaStart(ap, varg);
    points = Ch_VaArg(ap, const void*);
    center = Ch_VaArg(ap, CvPoint2D32f*);
    radius  = Ch_VaArg(ap, float*);
    cvMinEnclosingCircle(points, center, radius );
    Ch_VaEnd(ap);
}

EXPORTCH double cvMatchShapes_chdl(void *varg) {
    va_list ap;
    const void* contour1;
    const void* contour2;
    int method;
    double parameter;
    double retval;

    Ch_VaStart(ap, varg);
    contour1 = Ch_VaArg(ap, const void*);
    contour2 = Ch_VaArg(ap, const void*);
    method = Ch_VaArg(ap, int);
    parameter = Ch_VaArg(ap, double);
    retval = cvMatchShapes(contour1, contour2, method, parameter);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CvContourTree* cvCreateContourTree_chdl(void *varg) {
    va_list ap;
    CvSeq* contour;
    CvMemStorage* storage;
    double threshold;
    CvContourTree* retval;

    Ch_VaStart(ap, varg);
    contour = Ch_VaArg(ap, CvSeq*);
    storage = Ch_VaArg(ap, CvMemStorage*);
    threshold = Ch_VaArg(ap, double);
    retval = cvCreateContourTree(contour, storage, threshold);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CvSeq* cvContourFromContourTree_chdl(void *varg) {
    va_list ap;
    CvContourTree *tree;
    CvMemStorage* storage;
    CvTermCriteria criteria;
    CvSeq* retval;

    Ch_VaStart(ap, varg);
    tree = Ch_VaArg(ap, CvContourTree *);
    storage = Ch_VaArg(ap, CvMemStorage*);
    criteria = Ch_VaArg(ap, CvTermCriteria);
    retval = cvContourFromContourTree(tree, storage, criteria);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH double cvMatchContourTrees_chdl(void *varg) {
    va_list ap;
    CvContourTree *tree1;
    CvContourTree *tree2;
    int method;
    double threshold;
    double retval;

    Ch_VaStart(ap, varg);
    tree1 = Ch_VaArg(ap, CvContourTree *);
    tree2 = Ch_VaArg(ap, CvContourTree *);
    method = Ch_VaArg(ap, int);
    threshold = Ch_VaArg(ap, double);
    retval = cvMatchContourTrees(tree1, tree2, method, threshold);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvCalcPGH_chdl(void *varg) {
    va_list ap;
    const CvSeq* contour;
    CvHistogram* hist ;

    Ch_VaStart(ap, varg);
    contour = Ch_VaArg(ap, const CvSeq*);
    hist  = Ch_VaArg(ap, CvHistogram*);
    cvCalcPGH(contour, hist );
    Ch_VaEnd(ap);
}

EXPORTCH CvSeq* cvConvexHull2_chdl(void *varg) {
    va_list ap;
    const CvArr* input;
    void* hull_storage;
    int orientation;
    int return_points;
    CvSeq* retval;

    Ch_VaStart(ap, varg);
    input = Ch_VaArg(ap, const CvArr*);
    hull_storage = Ch_VaArg(ap, void*);
    orientation = Ch_VaArg(ap, int);
    return_points = Ch_VaArg(ap, int);
    retval = cvConvexHull2(input, hull_storage, orientation, return_points);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH int cvCheckContourConvexity_chdl(void *varg) {
    va_list ap;
    const CvArr* contour ;
    int retval;

    Ch_VaStart(ap, varg);
    contour  = Ch_VaArg(ap, const CvArr*);
    retval = cvCheckContourConvexity(contour );
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CvSeq* cvConvexityDefects_chdl(void *varg) {
    va_list ap;
    const CvArr* contour;
    const CvArr* convexhull;
    CvMemStorage* storage;
    CvSeq* retval;

    Ch_VaStart(ap, varg);
    contour = Ch_VaArg(ap, const CvArr*);
    convexhull = Ch_VaArg(ap, const CvArr*);
    storage = Ch_VaArg(ap, CvMemStorage*);
    retval = cvConvexityDefects(contour, convexhull, storage);
    Ch_VaEnd(ap);
    return retval;
}

/*
EXPORTCH CvBox2D cvFitEllipse2_chdl(void *varg) {
    va_list ap;
    const CvArr* points;
    CvBox2D retval;

    Ch_VaStart(ap, varg);
    points = Ch_VaArg(ap, const CvArr*);
    retval = cvFitEllipse2(points);
    Ch_VaEnd(ap);
    return retval;
}

*/
EXPORTCH CvHistogram* cvCreateHist_chdl(void *varg) {
    va_list ap;
    int dims;
    int* sizes;
    int type;
    float** ranges;
    int uniform;
    CvHistogram* retval;

    Ch_VaStart(ap, varg);
    dims = Ch_VaArg(ap, int);
    sizes = Ch_VaArg(ap, int*);
    type = Ch_VaArg(ap, int);
    ranges = Ch_VaArg(ap, float**);
    uniform = Ch_VaArg(ap, int);
    retval = cvCreateHist(dims, sizes, type, ranges, uniform);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvSetHistBinRanges_chdl(void *varg) {
    va_list ap;
    CvHistogram* hist;
    float** ranges;
    int uniform;

    Ch_VaStart(ap, varg);
    hist = Ch_VaArg(ap, CvHistogram*);
    ranges = Ch_VaArg(ap, float**);
    uniform = Ch_VaArg(ap, int);
    cvSetHistBinRanges(hist, ranges, uniform);
    Ch_VaEnd(ap);
}

EXPORTCH CvHistogram* cvMakeHistHeaderForArray_chdl(void *varg) {
    va_list ap;
    int dims;
    int* sizes;
    CvHistogram* hist;
    float* data;
    float** ranges;
    int uniform;
    CvHistogram* retval;

    Ch_VaStart(ap, varg);
    dims = Ch_VaArg(ap, int);
    sizes = Ch_VaArg(ap, int*);
    hist = Ch_VaArg(ap, CvHistogram*);
    data = Ch_VaArg(ap, float*);
    ranges = Ch_VaArg(ap, float**);
    uniform = Ch_VaArg(ap, int);
    retval = cvMakeHistHeaderForArray(dims, sizes, hist, data, ranges, uniform);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvReleaseHist_chdl(void *varg) {
    va_list ap;
    CvHistogram** hist;

    Ch_VaStart(ap, varg);
    hist = Ch_VaArg(ap, CvHistogram**);
    cvReleaseHist(hist);
    Ch_VaEnd(ap);
}

EXPORTCH void cvClearHist_chdl(void *varg) {
    va_list ap;
    CvHistogram* hist;

    Ch_VaStart(ap, varg);
    hist = Ch_VaArg(ap, CvHistogram*);
    cvClearHist(hist);
    Ch_VaEnd(ap);
}

EXPORTCH void cvGetMinMaxHistValue_chdl(void *varg) {
    va_list ap;
    const CvHistogram* hist;
    float* value_min;
    float* value_max;
    int* idx_min;
    int* idx_max;

    Ch_VaStart(ap, varg);
    hist = Ch_VaArg(ap, const CvHistogram*);
    value_min = Ch_VaArg(ap, float*);
    value_max = Ch_VaArg(ap, float*);
    idx_min = Ch_VaArg(ap, int*);
    idx_max = Ch_VaArg(ap, int*);
    cvGetMinMaxHistValue(hist, value_min, value_max, idx_min, idx_max);
    Ch_VaEnd(ap);
}

EXPORTCH void cvNormalizeHist_chdl(void *varg) {
    va_list ap;
    CvHistogram* hist;
    double factor ;

    Ch_VaStart(ap, varg);
    hist = Ch_VaArg(ap, CvHistogram*);
    factor  = Ch_VaArg(ap, double);
    cvNormalizeHist(hist, factor );
    Ch_VaEnd(ap);
}

EXPORTCH void cvThreshHist_chdl(void *varg) {
    va_list ap;
    CvHistogram* hist;
    double thresh ;

    Ch_VaStart(ap, varg);
    hist = Ch_VaArg(ap, CvHistogram*);
    thresh  = Ch_VaArg(ap, double);
    cvThreshHist(hist, thresh );
    Ch_VaEnd(ap);
}

EXPORTCH double cvCompareHist_chdl(void *varg) {
    va_list ap;
    const CvHistogram* hist1;
    const CvHistogram* hist2;
    int method;
    double retval;

    Ch_VaStart(ap, varg);
    hist1 = Ch_VaArg(ap, const CvHistogram*);
    hist2 = Ch_VaArg(ap, const CvHistogram*);
    method = Ch_VaArg(ap, int);
    retval = cvCompareHist(hist1, hist2, method);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvCopyHist_chdl(void *varg) {
    va_list ap;
    const CvHistogram* src;
    CvHistogram** dst;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvHistogram*);
    dst = Ch_VaArg(ap, CvHistogram**);
    cvCopyHist(src, dst);
    Ch_VaEnd(ap);
}

EXPORTCH void cvCalcBayesianProb_chdl(void *varg) {
    va_list ap;
    CvHistogram** src;
    int number;
    CvHistogram** dst;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, CvHistogram**);
    number = Ch_VaArg(ap, int);
    dst = Ch_VaArg(ap, CvHistogram**);
    cvCalcBayesianProb(src, number, dst);
    Ch_VaEnd(ap);
}

EXPORTCH void cvCalcArrHist_chdl(void *varg) {
    va_list ap;
    CvArr** arr;
    CvHistogram* hist;
    int doNotClear;
    const CvArr* mask;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, CvArr**);
    hist = Ch_VaArg(ap, CvHistogram*);
    doNotClear = Ch_VaArg(ap, int);
    mask = Ch_VaArg(ap, const CvArr*);
    cvCalcArrHist(arr, hist, doNotClear, mask);
    Ch_VaEnd(ap);
}

EXPORTCH void cvCalcArrBackProject_chdl(void *varg) {
    va_list ap;
    CvArr** img;
    CvArr* dst;
    const CvHistogram* hist ;

    Ch_VaStart(ap, varg);
    img = Ch_VaArg(ap, CvArr**);
    dst = Ch_VaArg(ap, CvArr*);
    hist  = Ch_VaArg(ap, const CvHistogram*);
    cvCalcArrBackProject(img, dst, hist );
    Ch_VaEnd(ap);
}

EXPORTCH void cvCalcArrBackProjectPatch_chdl(void *varg) {
    va_list ap;
    CvArr** img;
    CvArr* dst;
    CvSize range;
    CvHistogram* hist;
    int method;
    double normFactor;

    Ch_VaStart(ap, varg);
    img = Ch_VaArg(ap, CvArr**);
    dst = Ch_VaArg(ap, CvArr*);
    range = Ch_VaArg(ap, CvSize);
    hist = Ch_VaArg(ap, CvHistogram*);
    method = Ch_VaArg(ap, int);
    normFactor = Ch_VaArg(ap, double);
    cvCalcArrBackProjectPatch(img, dst, range, hist, method, normFactor);
    Ch_VaEnd(ap);
}

EXPORTCH void cvCalcProbDensity_chdl(void *varg) {
    va_list ap;
    const CvHistogram* hist;
    const CvHistogram* hist_mask;
    CvHistogram* hist_dens;
    double scale;

    Ch_VaStart(ap, varg);
    hist = Ch_VaArg(ap, const CvHistogram*);
    hist_mask = Ch_VaArg(ap, const CvHistogram*);
    hist_dens = Ch_VaArg(ap, CvHistogram*);
    scale = Ch_VaArg(ap, double);
    cvCalcProbDensity(hist, hist_mask, hist_dens, scale);
    Ch_VaEnd(ap);
}

EXPORTCH void cvSnakeImage_chdl(void *varg) {
    va_list ap;
    const IplImage* src;
    CvPoint* points;
    int length;
    float* alpha;
    float* beta;
    float* gamma;
    int coeffUsage;
    CvSize win;
    CvTermCriteria criteria;
    int calcGradient;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const IplImage*);
    points = Ch_VaArg(ap, CvPoint*);
    length = Ch_VaArg(ap, int);
    alpha = Ch_VaArg(ap, float*);
    beta = Ch_VaArg(ap, float*);
    gamma = Ch_VaArg(ap, float*);
    coeffUsage = Ch_VaArg(ap, int);
    win = Ch_VaArg(ap, CvSize);
    criteria = Ch_VaArg(ap, CvTermCriteria);
    calcGradient = Ch_VaArg(ap, int);
    cvSnakeImage(src, points, length, alpha, beta, gamma, coeffUsage, win, criteria, calcGradient);
    Ch_VaEnd(ap);
}

EXPORTCH void cvCalcImageHomography_chdl(void *varg) {
    va_list ap;
    float *line;
    CvPoint3D32f* center;
    float* intrinsic;
    float* homography;

    Ch_VaStart(ap, varg);
    line = Ch_VaArg(ap, float *);
    center = Ch_VaArg(ap, CvPoint3D32f*);
    intrinsic = Ch_VaArg(ap, float*);
    homography = Ch_VaArg(ap, float*);
    cvCalcImageHomography(line, center, intrinsic, homography);
    Ch_VaEnd(ap);
}

EXPORTCH void cvDistTransform_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvArr* dst;
    CvDisType disType;
    int maskSize;
    const float* mask;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    disType = Ch_VaArg(ap, CvDisType);
    maskSize = Ch_VaArg(ap, int);
    mask = Ch_VaArg(ap, const float*);
    cvDistTransform(src, dst, disType, maskSize, mask);
    Ch_VaEnd(ap);
}

EXPORTCH void cvThreshold_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvArr* dst;
    double thresh;
    double maxValue;
    int type;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    thresh = Ch_VaArg(ap, double);
    maxValue = Ch_VaArg(ap, double);
    type = Ch_VaArg(ap, int);
    cvThreshold(src, dst, thresh, maxValue, type);
    Ch_VaEnd(ap);
}

EXPORTCH void cvAdaptiveThreshold_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvArr* dst;
    double maxValue;
    int method;
    int type;
    int blockSize;
    double param1;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    maxValue = Ch_VaArg(ap, double);
    method = Ch_VaArg(ap, int);
    type = Ch_VaArg(ap, int);
    blockSize = Ch_VaArg(ap, int);
    param1 = Ch_VaArg(ap, double);
    cvAdaptiveThreshold(src, dst, maxValue, method, type, blockSize, param1);
    Ch_VaEnd(ap);
}

EXPORTCH void cvFloodFill_chdl(void *varg) {
    va_list ap;
    CvArr* arr;
    CvPoint seedPoint;
    double newVal;
    double loDiff;
    double upDiff;
    CvConnectedComp* comp;
    int flags;
    CvArr* mask;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, CvArr*);
    seedPoint = Ch_VaArg(ap, CvPoint);
    newVal = Ch_VaArg(ap, double);
    loDiff = Ch_VaArg(ap, double);
    upDiff = Ch_VaArg(ap, double);
    comp = Ch_VaArg(ap, CvConnectedComp*);
    flags = Ch_VaArg(ap, int);
    mask = Ch_VaArg(ap, CvArr*);
    cvFloodFill(arr, seedPoint, newVal, loDiff, upDiff, comp, flags, mask);
    Ch_VaEnd(ap);
}

EXPORTCH void cvCanny_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvArr* dst;
    double low_threshold;
    double high_threshold;
    int aperture_size;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    low_threshold = Ch_VaArg(ap, double);
    high_threshold = Ch_VaArg(ap, double);
    aperture_size = Ch_VaArg(ap, int);
    cvCanny(src, dst, low_threshold, high_threshold, aperture_size);
    Ch_VaEnd(ap);
}

EXPORTCH void cvPreCornerDetect_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvArr* dst;
    int aperture_size;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    dst = Ch_VaArg(ap, CvArr*);
    aperture_size = Ch_VaArg(ap, int);
    cvPreCornerDetect(src, dst, aperture_size);
    Ch_VaEnd(ap);
}

EXPORTCH void cvCornerEigenValsAndVecs_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvArr* eigenvv;
    int blockSize;
    int aperture_size;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    eigenvv = Ch_VaArg(ap, CvArr*);
    blockSize = Ch_VaArg(ap, int);
    aperture_size = Ch_VaArg(ap, int);
    cvCornerEigenValsAndVecs(src, eigenvv, blockSize, aperture_size);
    Ch_VaEnd(ap);
}

EXPORTCH void cvCornerMinEigenVal_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvArr* eigenval;
    int blockSize;
    int aperture_size;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    eigenval = Ch_VaArg(ap, CvArr*);
    blockSize = Ch_VaArg(ap, int);
    aperture_size = Ch_VaArg(ap, int);
    cvCornerMinEigenVal(src, eigenval, blockSize, aperture_size);
    Ch_VaEnd(ap);
}

EXPORTCH void cvFindCornerSubPix_chdl(void *varg) {
    va_list ap;
    const CvArr* src;
    CvPoint2D32f* corners;
    int count;
    CvSize win;
    CvSize zero_zone;
    CvTermCriteria criteria;

    Ch_VaStart(ap, varg);
    src = Ch_VaArg(ap, const CvArr*);
    corners = Ch_VaArg(ap, CvPoint2D32f*);
    count = Ch_VaArg(ap, int);
    win = Ch_VaArg(ap, CvSize);
    zero_zone = Ch_VaArg(ap, CvSize);
    criteria = Ch_VaArg(ap, CvTermCriteria);
    cvFindCornerSubPix(src, corners, count, win, zero_zone, criteria);
    Ch_VaEnd(ap);
}

EXPORTCH void cvGoodFeaturesToTrack_chdl(void *varg) {
    va_list ap;
    const CvArr* image;
    CvArr* eig_image;
    CvArr* temp_image;
    CvPoint2D32f* corners;
    int* corner_count;
    double quality_level;
    double min_distance;
    const CvArr* mask;

    Ch_VaStart(ap, varg);
    image = Ch_VaArg(ap, const CvArr*);
    eig_image = Ch_VaArg(ap, CvArr*);
    temp_image = Ch_VaArg(ap, CvArr*);
    corners = Ch_VaArg(ap, CvPoint2D32f*);
    corner_count = Ch_VaArg(ap, int*);
    quality_level = Ch_VaArg(ap, double);
    min_distance = Ch_VaArg(ap, double);
    mask = Ch_VaArg(ap, const CvArr*);
    cvGoodFeaturesToTrack(image, eig_image, temp_image, corners, corner_count, quality_level, min_distance, mask);
    Ch_VaEnd(ap);
}

EXPORTCH CvSeq* cvHoughLines2_chdl(void *varg) {
    va_list ap;
    CvArr* image;
    void* line_storage;
    int method;
    double rho;
    double theta;
    int threshold;
    double param1;
    double param2;
    CvSeq* retval;

    Ch_VaStart(ap, varg);
    image = Ch_VaArg(ap, CvArr*);
    line_storage = Ch_VaArg(ap, void*);
    method = Ch_VaArg(ap, int);
    rho = Ch_VaArg(ap, double);
    theta = Ch_VaArg(ap, double);
    threshold = Ch_VaArg(ap, int);
    param1 = Ch_VaArg(ap, double);
    param2 = Ch_VaArg(ap, double);
    retval = cvHoughLines2(image, line_storage, method, rho, theta, threshold, param1, param2);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvProject3D_chdl(void *varg) {
    va_list ap;
    CvPoint3D32f* points3D;
    int count;
    CvPoint2D32f* points2D;
    int xIndx;
    int yIndx;

    Ch_VaStart(ap, varg);
    points3D = Ch_VaArg(ap, CvPoint3D32f*);
    count = Ch_VaArg(ap, int);
    points2D = Ch_VaArg(ap, CvPoint2D32f*);
    xIndx = Ch_VaArg(ap, int);
    yIndx = Ch_VaArg(ap, int);
    cvProject3D(points3D, count, points2D, xIndx, yIndx);
    Ch_VaEnd(ap);
}

EXPORTCH void cvFitLine_chdl(void *varg) {
    va_list ap;
    const CvArr* points;
    CvDisType dist;
    double param;
    double reps;
    double aeps;
    float* line ;

    Ch_VaStart(ap, varg);
    points = Ch_VaArg(ap, const CvArr*);
    dist = Ch_VaArg(ap, CvDisType);
    param = Ch_VaArg(ap, double);
    reps = Ch_VaArg(ap, double);
    aeps = Ch_VaArg(ap, double);
    line  = Ch_VaArg(ap, float*);
    cvFitLine(points, dist, param, reps, aeps, line );
    Ch_VaEnd(ap);
}

EXPORTCH void cvCalcCovarMatrixEx_chdl(void *varg) {
    va_list ap;
    int nObjects;
    void* input;
    int ioFlags;
    int ioBufSize;
    uchar* buffer;
    void* userData;
    IplImage* avg;
    float* covarMatrix;

    Ch_VaStart(ap, varg);
    nObjects = Ch_VaArg(ap, int);
    input = Ch_VaArg(ap, void*);
    ioFlags = Ch_VaArg(ap, int);
    ioBufSize = Ch_VaArg(ap, int);
    buffer = Ch_VaArg(ap, uchar*);
    userData = Ch_VaArg(ap, void*);
    avg = Ch_VaArg(ap, IplImage*);
    covarMatrix = Ch_VaArg(ap, float*);
    cvCalcCovarMatrixEx(nObjects, input, ioFlags, ioBufSize, buffer, userData, avg, covarMatrix);
    Ch_VaEnd(ap);
}

EXPORTCH void cvCalcEigenObjects_chdl(void *varg) {
    va_list ap;
    int nObjects;
    void* input;
    void* output;
    int ioFlags;
    int ioBufSize;
    void* userData;
    CvTermCriteria* calcLimit;
    IplImage* avg;
    float* eigVals;

    Ch_VaStart(ap, varg);
    nObjects = Ch_VaArg(ap, int);
    input = Ch_VaArg(ap, void*);
    output = Ch_VaArg(ap, void*);
    ioFlags = Ch_VaArg(ap, int);
    ioBufSize = Ch_VaArg(ap, int);
    userData = Ch_VaArg(ap, void*);
    calcLimit = Ch_VaArg(ap, CvTermCriteria*);
    avg = Ch_VaArg(ap, IplImage*);
    eigVals = Ch_VaArg(ap, float*);
    cvCalcEigenObjects(nObjects, input, output, ioFlags, ioBufSize, userData, calcLimit, avg, eigVals);
    Ch_VaEnd(ap);
}

EXPORTCH double cvCalcDecompCoeff_chdl(void *varg) {
    va_list ap;
    IplImage* obj;
    IplImage* eigObj;
    IplImage* avg ;
    double retval;

    Ch_VaStart(ap, varg);
    obj = Ch_VaArg(ap, IplImage*);
    eigObj = Ch_VaArg(ap, IplImage*);
    avg  = Ch_VaArg(ap, IplImage*);
    retval = cvCalcDecompCoeff(obj, eigObj, avg );
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvEigenDecomposite_chdl(void *varg) {
    va_list ap;
    IplImage* obj;
    int nEigObjs;
    void* eigInput;
    int ioFlags;
    void* userData;
    IplImage* avg;
    float* coeffs ;

    Ch_VaStart(ap, varg);
    obj = Ch_VaArg(ap, IplImage*);
    nEigObjs = Ch_VaArg(ap, int);
    eigInput = Ch_VaArg(ap, void*);
    ioFlags = Ch_VaArg(ap, int);
    userData = Ch_VaArg(ap, void*);
    avg = Ch_VaArg(ap, IplImage*);
    coeffs  = Ch_VaArg(ap, float*);
    cvEigenDecomposite(obj, nEigObjs, eigInput, ioFlags, userData, avg, coeffs );
    Ch_VaEnd(ap);
}

EXPORTCH void cvEigenProjection_chdl(void *varg) {
    va_list ap;
    void* eigInput;
    int nEigObjs;
    int ioFlags;
    void* userData;
    float* coeffs;
    IplImage* avg;
    IplImage* proj;

    Ch_VaStart(ap, varg);
    eigInput = Ch_VaArg(ap, void*);
    nEigObjs = Ch_VaArg(ap, int);
    ioFlags = Ch_VaArg(ap, int);
    userData = Ch_VaArg(ap, void*);
    coeffs = Ch_VaArg(ap, float*);
    avg = Ch_VaArg(ap, IplImage*);
    proj = Ch_VaArg(ap, IplImage*);
    cvEigenProjection(eigInput, nEigObjs, ioFlags, userData, coeffs, avg, proj);
    Ch_VaEnd(ap);
}

EXPORTCH CvEHMM* cvCreate2DHMM_chdl(void *varg) {
    va_list ap;
    int* stateNumber;
    int* numMix;
    int obsSize ;
    CvEHMM* retval;

    Ch_VaStart(ap, varg);
    stateNumber = Ch_VaArg(ap, int*);
    numMix = Ch_VaArg(ap, int*);
    obsSize  = Ch_VaArg(ap, int);
    retval = cvCreate2DHMM(stateNumber, numMix, obsSize );
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvRelease2DHMM_chdl(void *varg) {
    va_list ap;
    CvEHMM** hmm ;

    Ch_VaStart(ap, varg);
    hmm  = Ch_VaArg(ap, CvEHMM**);
    cvRelease2DHMM(hmm );
    Ch_VaEnd(ap);
}

EXPORTCH CvImgObsInfo* cvCreateObsInfo_chdl(void *varg) {
    va_list ap;
    CvSize numObs;
    int obsSize ;
    CvImgObsInfo* retval;

    Ch_VaStart(ap, varg);
    numObs = Ch_VaArg(ap, CvSize);
    obsSize  = Ch_VaArg(ap, int);
    retval = cvCreateObsInfo(numObs, obsSize );
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvReleaseObsInfo_chdl(void *varg) {
    va_list ap;
    CvImgObsInfo** obs_info ;

    Ch_VaStart(ap, varg);
    obs_info  = Ch_VaArg(ap, CvImgObsInfo**);
    cvReleaseObsInfo(obs_info );
    Ch_VaEnd(ap);
}

EXPORTCH void cvImgToObs_DCT_chdl(void *varg) {
    va_list ap;
    const CvArr* arr;
    float* obs;
    CvSize dctSize;
    CvSize obsSize;
    CvSize delta ;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, const CvArr*);
    obs = Ch_VaArg(ap, float*);
    dctSize = Ch_VaArg(ap, CvSize);
    obsSize = Ch_VaArg(ap, CvSize);
    delta  = Ch_VaArg(ap, CvSize);
    cvImgToObs_DCT(arr, obs, dctSize, obsSize, delta );
    Ch_VaEnd(ap);
}

EXPORTCH void cvUniformImgSegm_chdl(void *varg) {
    va_list ap;
    CvImgObsInfo* obs_info;
    CvEHMM* ehmm ;

    Ch_VaStart(ap, varg);
    obs_info = Ch_VaArg(ap, CvImgObsInfo*);
    ehmm  = Ch_VaArg(ap, CvEHMM*);
    cvUniformImgSegm(obs_info, ehmm );
    Ch_VaEnd(ap);
}

EXPORTCH void cvInitMixSegm_chdl(void *varg) {
    va_list ap;
    CvImgObsInfo** obs_info_array;
    int num_img;
    CvEHMM* hmm ;

    Ch_VaStart(ap, varg);
    obs_info_array = Ch_VaArg(ap, CvImgObsInfo**);
    num_img = Ch_VaArg(ap, int);
    hmm  = Ch_VaArg(ap, CvEHMM*);
    cvInitMixSegm(obs_info_array, num_img, hmm );
    Ch_VaEnd(ap);
}

EXPORTCH void cvEstimateHMMStateParams_chdl(void *varg) {
    va_list ap;
    CvImgObsInfo** obs_info_array;
    int num_img;
    CvEHMM* hmm ;

    Ch_VaStart(ap, varg);
    obs_info_array = Ch_VaArg(ap, CvImgObsInfo**);
    num_img = Ch_VaArg(ap, int);
    hmm  = Ch_VaArg(ap, CvEHMM*);
    cvEstimateHMMStateParams(obs_info_array, num_img, hmm );
    Ch_VaEnd(ap);
}

EXPORTCH void cvEstimateTransProb_chdl(void *varg) {
    va_list ap;
    CvImgObsInfo** obs_info_array;
    int num_img;
    CvEHMM* hmm ;

    Ch_VaStart(ap, varg);
    obs_info_array = Ch_VaArg(ap, CvImgObsInfo**);
    num_img = Ch_VaArg(ap, int);
    hmm  = Ch_VaArg(ap, CvEHMM*);
    cvEstimateTransProb(obs_info_array, num_img, hmm );
    Ch_VaEnd(ap);
}

EXPORTCH void cvEstimateObsProb_chdl(void *varg) {
    va_list ap;
    CvImgObsInfo* obs_info;
    CvEHMM* hmm ;

    Ch_VaStart(ap, varg);
    obs_info = Ch_VaArg(ap, CvImgObsInfo*);
    hmm  = Ch_VaArg(ap, CvEHMM*);
    cvEstimateObsProb(obs_info, hmm );
    Ch_VaEnd(ap);
}

EXPORTCH float cvEViterbi_chdl(void *varg) {
    va_list ap;
    CvImgObsInfo* obs_info;
    CvEHMM* hmm ;
    float retval;

    Ch_VaStart(ap, varg);
    obs_info = Ch_VaArg(ap, CvImgObsInfo*);
    hmm  = Ch_VaArg(ap, CvEHMM*);
    retval = cvEViterbi(obs_info, hmm );
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvMixSegmL2_chdl(void *varg) {
    va_list ap;
    CvImgObsInfo** obs_info_array;
    int num_img;
    CvEHMM* hmm ;

    Ch_VaStart(ap, varg);
    obs_info_array = Ch_VaArg(ap, CvImgObsInfo**);
    num_img = Ch_VaArg(ap, int);
    hmm  = Ch_VaArg(ap, CvEHMM*);
    cvMixSegmL2(obs_info_array, num_img, hmm );
    Ch_VaEnd(ap);
}

EXPORTCH void cvKMeans2_chdl(void *varg) {
    va_list ap;
    const CvArr* samples;
    int cluster_count;
    CvArr* cluster_idx;
    CvTermCriteria termcrit ;

    Ch_VaStart(ap, varg);
    samples = Ch_VaArg(ap, const CvArr*);
    cluster_count = Ch_VaArg(ap, int);
    cluster_idx = Ch_VaArg(ap, CvArr*);
    termcrit  = Ch_VaArg(ap, CvTermCriteria);
    cvKMeans2(samples, cluster_count, cluster_idx, termcrit );
    Ch_VaEnd(ap);
}

EXPORTCH void cvUnDistortOnce_chdl(void *varg) {
    va_list ap;
    const CvArr* srcImage;
    CvArr* dstImage;
    const float* intrMatrix;
    const float* distCoeffs;
    int interpolate;

    Ch_VaStart(ap, varg);
    srcImage = Ch_VaArg(ap, const CvArr*);
    dstImage = Ch_VaArg(ap, CvArr*);
    intrMatrix = Ch_VaArg(ap, const float*);
    distCoeffs = Ch_VaArg(ap, const float*);
    interpolate = Ch_VaArg(ap, int);
    cvUnDistortOnce(srcImage, dstImage, intrMatrix, distCoeffs, interpolate);
    Ch_VaEnd(ap);
}

EXPORTCH void cvUnDistortInit_chdl(void *varg) {
    va_list ap;
    const CvArr* srcImage;
    CvArr* undistMap;
    const float* intrMatrix;
    const float* distCoeffs;
    int interpolate;

    Ch_VaStart(ap, varg);
    srcImage = Ch_VaArg(ap, const CvArr*);
    undistMap = Ch_VaArg(ap, CvArr*);
    intrMatrix = Ch_VaArg(ap, const float*);
    distCoeffs = Ch_VaArg(ap, const float*);
    interpolate = Ch_VaArg(ap, int);
    cvUnDistortInit(srcImage, undistMap, intrMatrix, distCoeffs, interpolate);
    Ch_VaEnd(ap);
}

EXPORTCH void cvUnDistort_chdl(void *varg) {
    va_list ap;
    const CvArr* srcImage;
    CvArr* dstImage;
    const CvArr* undistMap;
    int interpolate;

    Ch_VaStart(ap, varg);
    srcImage = Ch_VaArg(ap, const CvArr*);
    dstImage = Ch_VaArg(ap, CvArr*);
    undistMap = Ch_VaArg(ap, const CvArr*);
    interpolate = Ch_VaArg(ap, int);
    cvUnDistort(srcImage, dstImage, undistMap, interpolate);
    Ch_VaEnd(ap);
}

EXPORTCH void cvConvertMap_chdl(void *varg) {
    va_list ap;
    const CvArr* srcImage;
    const CvArr* flUndistMap;
    CvArr* undistMap;
    int iterpolate;

    Ch_VaStart(ap, varg);
    srcImage = Ch_VaArg(ap, const CvArr*);
    flUndistMap = Ch_VaArg(ap, const CvArr*);
    undistMap = Ch_VaArg(ap, CvArr*);
    iterpolate = Ch_VaArg(ap, int);
    cvConvertMap(srcImage, flUndistMap, undistMap, iterpolate);
    Ch_VaEnd(ap);
}

EXPORTCH void cvCalibrateCamera_chdl(void *varg) {
    va_list ap;
    int numImages;
    int* numPoints;
    CvSize imageSize;
    CvPoint2D32f* imagePoints32f;
    CvPoint3D32f* objectPoints32f;
    CvVect32f distortion32f;
    CvMatr32f cameraMatrix32f;
    CvVect32f transVects32f;
    CvMatr32f rotMatrs32f;
    int useIntrinsicGuess;

    Ch_VaStart(ap, varg);
    numImages = Ch_VaArg(ap, int);
    numPoints = Ch_VaArg(ap, int*);
    imageSize = Ch_VaArg(ap, CvSize);
    imagePoints32f = Ch_VaArg(ap, CvPoint2D32f*);
    objectPoints32f = Ch_VaArg(ap, CvPoint3D32f*);
    distortion32f = Ch_VaArg(ap, CvVect32f);
    cameraMatrix32f = Ch_VaArg(ap, CvMatr32f);
    transVects32f = Ch_VaArg(ap, CvVect32f);
    rotMatrs32f = Ch_VaArg(ap, CvMatr32f);
    useIntrinsicGuess = Ch_VaArg(ap, int);
    cvCalibrateCamera(numImages, numPoints, imageSize, imagePoints32f, objectPoints32f, distortion32f, cameraMatrix32f, transVects32f, rotMatrs32f, useIntrinsicGuess);
    Ch_VaEnd(ap);
}

EXPORTCH void cvCalibrateCamera_64d_chdl(void *varg) {
    va_list ap;
    int numImages;
    int* numPoints;
    CvSize imageSize;
    CvPoint2D64d* imagePoints;
    CvPoint3D64d* objectPoints;
    CvVect64d distortion;
    CvMatr64d cameraMatrix;
    CvVect64d transVects;
    CvMatr64d rotMatrs;
    int useIntrinsicGuess;

    Ch_VaStart(ap, varg);
    numImages = Ch_VaArg(ap, int);
    numPoints = Ch_VaArg(ap, int*);
    imageSize = Ch_VaArg(ap, CvSize);
    imagePoints = Ch_VaArg(ap, CvPoint2D64d*);
    objectPoints = Ch_VaArg(ap, CvPoint3D64d*);
    distortion = Ch_VaArg(ap, CvVect64d);
    cameraMatrix = Ch_VaArg(ap, CvMatr64d);
    transVects = Ch_VaArg(ap, CvVect64d);
    rotMatrs = Ch_VaArg(ap, CvMatr64d);
    useIntrinsicGuess = Ch_VaArg(ap, int);
    cvCalibrateCamera_64d(numImages, numPoints, imageSize, imagePoints, objectPoints, distortion, cameraMatrix, transVects, rotMatrs, useIntrinsicGuess);
    Ch_VaEnd(ap);
}

EXPORTCH void cvFindExtrinsicCameraParams_chdl(void *varg) {
    va_list ap;
    int numPoints;
    CvSize imageSize;
    CvPoint2D32f* imagePoints32f;
    CvPoint3D32f* objectPoints32f;
    CvVect32f focalLength32f;
    CvPoint2D32f principalPoint32f;
    CvVect32f distortion32f;
    CvVect32f rotVect32f;
    CvVect32f transVect32f;

    Ch_VaStart(ap, varg);
    numPoints = Ch_VaArg(ap, int);
    imageSize = Ch_VaArg(ap, CvSize);
    imagePoints32f = Ch_VaArg(ap, CvPoint2D32f*);
    objectPoints32f = Ch_VaArg(ap, CvPoint3D32f*);
    focalLength32f = Ch_VaArg(ap, CvVect32f);
    principalPoint32f = Ch_VaArg(ap, CvPoint2D32f);
    distortion32f = Ch_VaArg(ap, CvVect32f);
    rotVect32f = Ch_VaArg(ap, CvVect32f);
    transVect32f = Ch_VaArg(ap, CvVect32f);
    cvFindExtrinsicCameraParams(numPoints, imageSize, imagePoints32f, objectPoints32f, focalLength32f, principalPoint32f, distortion32f, rotVect32f, transVect32f);
    Ch_VaEnd(ap);
}

EXPORTCH void cvFindExtrinsicCameraParams_64d_chdl(void *varg) {
    va_list ap;
    int numPoints;
    CvSize imageSize;
    CvPoint2D64d* imagePoints;
    CvPoint3D64d* objectPoints;
    CvVect64d focalLength;
    CvPoint2D64d principalPoint;
    CvVect64d distortion;
    CvVect64d rotVect;
    CvVect64d transVect;

    Ch_VaStart(ap, varg);
    numPoints = Ch_VaArg(ap, int);
    imageSize = Ch_VaArg(ap, CvSize);
    imagePoints = Ch_VaArg(ap, CvPoint2D64d*);
    objectPoints = Ch_VaArg(ap, CvPoint3D64d*);
    focalLength = Ch_VaArg(ap, CvVect64d);
    principalPoint = Ch_VaArg(ap, CvPoint2D64d);
    distortion = Ch_VaArg(ap, CvVect64d);
    rotVect = Ch_VaArg(ap, CvVect64d);
    transVect = Ch_VaArg(ap, CvVect64d);
    cvFindExtrinsicCameraParams_64d(numPoints, imageSize, imagePoints, objectPoints, focalLength, principalPoint, distortion, rotVect, transVect);
    Ch_VaEnd(ap);
}

EXPORTCH void cvRodrigues_chdl(void *varg) {
    va_list ap;
    CvMat* rotMatrix;
    CvMat* rotVector;
    CvMat* jacobian;
    int convType;

    Ch_VaStart(ap, varg);
    rotMatrix = Ch_VaArg(ap, CvMat*);
    rotVector = Ch_VaArg(ap, CvMat*);
    jacobian = Ch_VaArg(ap, CvMat*);
    convType = Ch_VaArg(ap, int);
    cvRodrigues(rotMatrix, rotVector, jacobian, convType);
    Ch_VaEnd(ap);
}

EXPORTCH void cvProjectPoints_chdl(void *varg) {
    va_list ap;
    int numPoints;
    CvPoint3D64d* objectPoints;
    CvVect64d rotVect;
    CvVect64d transVect;
    CvVect64d focalLength;
    CvPoint2D64d principalPoint;
    CvVect64d distortion;
    CvPoint2D64d* imagePoints;
    CvVect64d derivPointsRot;
    CvVect64d derivPointsTrans;
    CvVect64d derivPointsFocal;
    CvVect64d derivPointsPrincipal;
    CvVect64d derivPointsDistort;

    Ch_VaStart(ap, varg);
    numPoints = Ch_VaArg(ap, int);
    objectPoints = Ch_VaArg(ap, CvPoint3D64d*);
    rotVect = Ch_VaArg(ap, CvVect64d);
    transVect = Ch_VaArg(ap, CvVect64d);
    focalLength = Ch_VaArg(ap, CvVect64d);
    principalPoint = Ch_VaArg(ap, CvPoint2D64d);
    distortion = Ch_VaArg(ap, CvVect64d);
    imagePoints = Ch_VaArg(ap, CvPoint2D64d*);
    derivPointsRot = Ch_VaArg(ap, CvVect64d);
    derivPointsTrans = Ch_VaArg(ap, CvVect64d);
    derivPointsFocal = Ch_VaArg(ap, CvVect64d);
    derivPointsPrincipal = Ch_VaArg(ap, CvVect64d);
    derivPointsDistort = Ch_VaArg(ap, CvVect64d);
    cvProjectPoints(numPoints, objectPoints, rotVect, transVect, focalLength, principalPoint, distortion, imagePoints, derivPointsRot, derivPointsTrans, derivPointsFocal, derivPointsPrincipal, derivPointsDistort);
    Ch_VaEnd(ap);
}

EXPORTCH void cvProjectPointsSimple_chdl(void *varg) {
    va_list ap;
    int numPoints;
    CvPoint3D64d *objectPoints;
    CvVect64d rotMatr;
    CvVect64d transVect;
    CvMatr64d cameraMatrix;
    CvVect64d distortion;
    CvPoint2D64d* imagePoints;

    Ch_VaStart(ap, varg);
    numPoints = Ch_VaArg(ap, int);
    objectPoints = Ch_VaArg(ap, CvPoint3D64d *);
    rotMatr = Ch_VaArg(ap, CvVect64d);
    transVect = Ch_VaArg(ap, CvVect64d);
    cameraMatrix = Ch_VaArg(ap, CvMatr64d);
    distortion = Ch_VaArg(ap, CvVect64d);
    imagePoints = Ch_VaArg(ap, CvPoint2D64d*);
    cvProjectPointsSimple(numPoints, objectPoints, rotMatr, transVect, cameraMatrix, distortion, imagePoints);
    Ch_VaEnd(ap);
}

EXPORTCH int cvFindChessBoardCornerGuesses_chdl(void *varg) {
    va_list ap;
    const CvArr* arr;
    CvArr* thresh;
    CvMemStorage* storage;
    CvSize etalon_size;
    CvPoint2D32f* corners;
    int *corner_count;
    int retval;

    Ch_VaStart(ap, varg);
    arr = Ch_VaArg(ap, const CvArr*);
    thresh = Ch_VaArg(ap, CvArr*);
    storage = Ch_VaArg(ap, CvMemStorage*);
    etalon_size = Ch_VaArg(ap, CvSize);
    corners = Ch_VaArg(ap, CvPoint2D32f*);
    corner_count = Ch_VaArg(ap, int *);
    retval = cvFindChessBoardCornerGuesses(arr, thresh, storage, etalon_size, corners, corner_count);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CvPOSITObject* cvCreatePOSITObject_chdl(void *varg) {
    va_list ap;
    CvPoint3D32f* points;
    int numPoints ;
    CvPOSITObject* retval;

    Ch_VaStart(ap, varg);
    points = Ch_VaArg(ap, CvPoint3D32f*);
    numPoints  = Ch_VaArg(ap, int);
    retval = cvCreatePOSITObject(points, numPoints );
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvPOSIT_chdl(void *varg) {
    va_list ap;
    CvPOSITObject* pObject;
    CvPoint2D32f* imagePoints;
    double focalLength;
    CvTermCriteria criteria;
    CvMatr32f rotation;
    CvVect32f translation;

    Ch_VaStart(ap, varg);
    pObject = Ch_VaArg(ap, CvPOSITObject*);
    imagePoints = Ch_VaArg(ap, CvPoint2D32f*);
    focalLength = Ch_VaArg(ap, double);
    criteria = Ch_VaArg(ap, CvTermCriteria);
    rotation = Ch_VaArg(ap, CvMatr32f);
    translation = Ch_VaArg(ap, CvVect32f);
    cvPOSIT(pObject, imagePoints, focalLength, criteria, rotation, translation);
    Ch_VaEnd(ap);
}

EXPORTCH void cvReleasePOSITObject_chdl(void *varg) {
    va_list ap;
    CvPOSITObject** ppObject ;

    Ch_VaStart(ap, varg);
    ppObject  = Ch_VaArg(ap, CvPOSITObject**);
    cvReleasePOSITObject(ppObject );
    Ch_VaEnd(ap);
}

EXPORTCH void cvMake2DPoints_chdl(void *varg) {
    va_list ap;
    CvMat* srcPoint;
    CvMat* dstPoint;

    Ch_VaStart(ap, varg);
    srcPoint = Ch_VaArg(ap, CvMat* );
    dstPoint = Ch_VaArg(ap, CvMat* );
    cvMake2DPoints(srcPoint, dstPoint);
    Ch_VaEnd(ap);
}

EXPORTCH void cvMake3DPoints_chdl(void *varg) {
    va_list ap;
    CvMat* srcPoint;
    CvMat* dstPoint;

    Ch_VaStart(ap, varg);
    srcPoint = Ch_VaArg(ap, CvMat* );
    dstPoint = Ch_VaArg(ap, CvMat* );
    cvMake3DPoints(srcPoint, dstPoint);
    Ch_VaEnd(ap);
}

EXPORTCH int cvSolveCubic_chdl(void *varg) {
    va_list ap;
    CvMat* coeffs;
    CvMat* result;
    int retval;

    Ch_VaStart(ap, varg);
    coeffs = Ch_VaArg(ap, CvMat* );
    result = Ch_VaArg(ap, CvMat* );
    retval = cvSolveCubic(coeffs, result);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH int cvFindFundamentalMat_chdl(void *varg) {
    va_list ap;
    CvMat* points1;
    CvMat* points2;
    CvMat* fundMatr;
    int method;
    double param1;
    double param2;
    CvMat* status ;
    int retval;

    Ch_VaStart(ap, varg);
    points1 = Ch_VaArg(ap, CvMat*);
    points2 = Ch_VaArg(ap, CvMat*);
    fundMatr = Ch_VaArg(ap, CvMat*);
    method = Ch_VaArg(ap, int);
    param1 = Ch_VaArg(ap, double);
    param2 = Ch_VaArg(ap, double);
    status  = Ch_VaArg(ap, CvMat*);
    retval = cvFindFundamentalMat(points1, points2, fundMatr, method, param1, param2, status );
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvComputeCorrespondEpilines_chdl(void *varg) {
    va_list ap;
    CvMat* points;
    int pointImageID;
    CvMat* fundMatr;
    CvMat* corrLines;

    Ch_VaStart(ap, varg);
    points = Ch_VaArg(ap, CvMat*);
    pointImageID = Ch_VaArg(ap, int);
    fundMatr = Ch_VaArg(ap, CvMat*);
    corrLines = Ch_VaArg(ap, CvMat*);
    cvComputeCorrespondEpilines(points, pointImageID, fundMatr, corrLines);
    Ch_VaEnd(ap);
}

EXPORTCH void cvMakeScanlines_chdl(void *varg) {
    va_list ap;
    const CvMatrix3* matrix;
    CvSize imgSize;
    int* scanlines_1;
    int* scanlines_2;
    int* lens_1;
    int* lens_2;
    int* numlines;

    Ch_VaStart(ap, varg);
    matrix = Ch_VaArg(ap, const CvMatrix3*);
    imgSize = Ch_VaArg(ap, CvSize);
    scanlines_1 = Ch_VaArg(ap, int*);
    scanlines_2 = Ch_VaArg(ap, int*);
    lens_1 = Ch_VaArg(ap, int*);
    lens_2 = Ch_VaArg(ap, int*);
    numlines = Ch_VaArg(ap, int*);
    cvMakeScanlines(matrix, imgSize, scanlines_1, scanlines_2, lens_1, lens_2, numlines);
    Ch_VaEnd(ap);
}

EXPORTCH void cvPreWarpImage_chdl(void *varg) {
    va_list ap;
    int numLines;
    IplImage* img;
    uchar* dst;
    int* dst_nums;
    int* scanlines;

    Ch_VaStart(ap, varg);
    numLines = Ch_VaArg(ap, int);
    img = Ch_VaArg(ap, IplImage*);
    dst = Ch_VaArg(ap, uchar*);
    dst_nums = Ch_VaArg(ap, int*);
    scanlines = Ch_VaArg(ap, int*);
    cvPreWarpImage(numLines, img, dst, dst_nums, scanlines);
    Ch_VaEnd(ap);
}

EXPORTCH void cvFindRuns_chdl(void *varg) {
    va_list ap;
    int numLines;
    uchar* prewarp_1;
    uchar* prewarp_2;
    int* line_lens_1;
    int* line_lens_2;
    int* runs_1;
    int* runs_2;
    int* num_runs_1;
    int* num_runs_2;

    Ch_VaStart(ap, varg);
    numLines = Ch_VaArg(ap, int);
    prewarp_1 = Ch_VaArg(ap, uchar*);
    prewarp_2 = Ch_VaArg(ap, uchar*);
    line_lens_1 = Ch_VaArg(ap, int*);
    line_lens_2 = Ch_VaArg(ap, int*);
    runs_1 = Ch_VaArg(ap, int*);
    runs_2 = Ch_VaArg(ap, int*);
    num_runs_1 = Ch_VaArg(ap, int*);
    num_runs_2 = Ch_VaArg(ap, int*);
    cvFindRuns(numLines, prewarp_1, prewarp_2, line_lens_1, line_lens_2, runs_1, runs_2, num_runs_1, num_runs_2);
    Ch_VaEnd(ap);
}

EXPORTCH void cvDynamicCorrespondMulti_chdl(void *varg) {
    va_list ap;
    int lines;
    int* first;
    int* first_runs;
    int* second;
    int* second_runs;
    int* first_corr;
    int* second_corr;

    Ch_VaStart(ap, varg);
    lines = Ch_VaArg(ap, int);
    first = Ch_VaArg(ap, int*);
    first_runs = Ch_VaArg(ap, int*);
    second = Ch_VaArg(ap, int*);
    second_runs = Ch_VaArg(ap, int*);
    first_corr = Ch_VaArg(ap, int*);
    second_corr = Ch_VaArg(ap, int*);
    cvDynamicCorrespondMulti(lines, first, first_runs, second, second_runs, first_corr, second_corr);
    Ch_VaEnd(ap);
}

EXPORTCH void cvMakeAlphaScanlines_chdl(void *varg) {
    va_list ap;
    int* scanlines_1;
    int* scanlines_2;
    int* scanlines_a;
    int* lens;
    int numlines;
    float alpha;

    Ch_VaStart(ap, varg);
    scanlines_1 = Ch_VaArg(ap, int*);
    scanlines_2 = Ch_VaArg(ap, int*);
    scanlines_a = Ch_VaArg(ap, int*);
    lens = Ch_VaArg(ap, int*);
    numlines = Ch_VaArg(ap, int);
    alpha = Ch_VaArg(ap, float);
    cvMakeAlphaScanlines(scanlines_1, scanlines_2, scanlines_a, lens, numlines, alpha);
    Ch_VaEnd(ap);
}

EXPORTCH void cvMorphEpilinesMulti_chdl(void *varg) {
    va_list ap;
    int lines;
    uchar* first_pix;
    int* first_num;
    uchar* second_pix;
    int* second_num;
    uchar* dst_pix;
    int* dst_num;
    float alpha;
    int* first;
    int* first_runs;
    int* second;
    int* second_runs;
    int* first_corr;
    int* second_corr;

    Ch_VaStart(ap, varg);
    lines = Ch_VaArg(ap, int);
    first_pix = Ch_VaArg(ap, uchar*);
    first_num = Ch_VaArg(ap, int*);
    second_pix = Ch_VaArg(ap, uchar*);
    second_num = Ch_VaArg(ap, int*);
    dst_pix = Ch_VaArg(ap, uchar*);
    dst_num = Ch_VaArg(ap, int*);
    alpha = Ch_VaArg(ap, float);
    first = Ch_VaArg(ap, int*);
    first_runs = Ch_VaArg(ap, int*);
    second = Ch_VaArg(ap, int*);
    second_runs = Ch_VaArg(ap, int*);
    first_corr = Ch_VaArg(ap, int*);
    second_corr = Ch_VaArg(ap, int*);
    cvMorphEpilinesMulti(lines, first_pix, first_num, second_pix, second_num, dst_pix, dst_num, alpha, first, first_runs, second, second_runs, first_corr, second_corr);
    Ch_VaEnd(ap);
}

EXPORTCH void cvPostWarpImage_chdl(void *varg) {
    va_list ap;
    int numLines;
    uchar* src;
    int* src_nums;
    IplImage* img;
    int* scanlines;

    Ch_VaStart(ap, varg);
    numLines = Ch_VaArg(ap, int);
    src = Ch_VaArg(ap, uchar*);
    src_nums = Ch_VaArg(ap, int*);
    img = Ch_VaArg(ap, IplImage*);
    scanlines = Ch_VaArg(ap, int*);
    cvPostWarpImage(numLines, src, src_nums, img, scanlines);
    Ch_VaEnd(ap);
}

EXPORTCH void cvDeleteMoire_chdl(void *varg) {
    va_list ap;
    IplImage* img;

    Ch_VaStart(ap, varg);
    img = Ch_VaArg(ap, IplImage*);
    cvDeleteMoire(img);
    Ch_VaEnd(ap);
}

EXPORTCH int cvLoadPrimitives_chdl(void *varg) {
    va_list ap;
    const char* proc_type;
    int retval;

    Ch_VaStart(ap, varg);
    proc_type = Ch_VaArg(ap, const char*);
    retval = cvLoadPrimitives(proc_type);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH int cvFillInternalFuncsTable_chdl(void *varg) {
    va_list ap;
    void* table;
    int retval;

    Ch_VaStart(ap, varg);
    table = Ch_VaArg(ap, void* );
    retval = cvFillInternalFuncsTable(table);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvGetLibraryInfo_chdl(void *varg) {
    va_list ap;
    const char** version;
    int* loaded;
    const char** loaded_modules;

    Ch_VaStart(ap, varg);
    version = Ch_VaArg(ap, const char**);
    loaded = Ch_VaArg(ap, int*);
    loaded_modules = Ch_VaArg(ap, const char**);
    cvGetLibraryInfo(version, loaded, loaded_modules);
    Ch_VaEnd(ap);
}

EXPORTCH CVStatus cvGetErrStatus_chdl(void *varg) {
    CVStatus retval;
    retval = cvGetErrStatus();
    return retval;
}

EXPORTCH void cvSetErrStatus_chdl(void *varg) {
    va_list ap;
    CVStatus status ;

    Ch_VaStart(ap, varg);
    status  = Ch_VaArg(ap, CVStatus);
    cvSetErrStatus(status );
    Ch_VaEnd(ap);
}

EXPORTCH int cvGetErrMode_chdl(void *varg) {
    int retval;
    retval = cvGetErrMode();
    return retval;
}

EXPORTCH void cvSetErrMode_chdl(void *varg) {
    va_list ap;
    int mode;

    Ch_VaStart(ap, varg);
    mode = Ch_VaArg(ap, int);
    cvSetErrMode(mode);
    Ch_VaEnd(ap);
}

EXPORTCH CVStatus cvError_chdl(void *varg) {
    va_list ap;
    CVStatus code;
    const char *func;
    const char *context;
    const char *file;
    int line;
    CVStatus retval;

    Ch_VaStart(ap, varg);
    code = Ch_VaArg(ap, CVStatus);
    func = Ch_VaArg(ap, const char *);
    context = Ch_VaArg(ap, const char *);
    file = Ch_VaArg(ap, const char *);
    line = Ch_VaArg(ap, int);
    retval = cvError(code, func, context, file, line);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH const char* cvErrorStr_chdl(void *varg) {
    va_list ap;
    CVStatus status ;
    const char* retval;

    Ch_VaStart(ap, varg);
    status  = Ch_VaArg(ap, CVStatus);
    retval = cvErrorStr(status );
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CVErrorCallBack cvRedirectError_chdl(void *varg) {
    va_list ap;
    CVErrorCallBack cvErrorFunc;
    CVErrorCallBack retval;

    Ch_VaStart(ap, varg);
    cvErrorFunc = Ch_VaArg(ap, CVErrorCallBack);
    retval = cvRedirectError(cvErrorFunc);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CVStatus cvNulDevReport_chdl(void *varg) {
    va_list ap;
    CVStatus status;
    const char *funcName;
    const char *context;
    const char *file;
    int line;
    CVStatus retval;

    Ch_VaStart(ap, varg);
    status = Ch_VaArg(ap, CVStatus);
    funcName = Ch_VaArg(ap, const char *);
    context = Ch_VaArg(ap, const char *);
    file = Ch_VaArg(ap, const char *);
    line = Ch_VaArg(ap, int);
    retval = cvNulDevReport(status, funcName, context, file, line);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CVStatus cvStdErrReport_chdl(void *varg) {
    va_list ap;
    CVStatus status;
    const char *funcName;
    const char *context;
    const char *file;
    int line;
    CVStatus retval;

    Ch_VaStart(ap, varg);
    status = Ch_VaArg(ap, CVStatus);
    funcName = Ch_VaArg(ap, const char *);
    context = Ch_VaArg(ap, const char *);
    file = Ch_VaArg(ap, const char *);
    line = Ch_VaArg(ap, int);
    retval = cvStdErrReport(status, funcName, context, file, line);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CVStatus cvGuiBoxReport_chdl(void *varg) {
    va_list ap;
    CVStatus status;
    const char *funcName;
    const char *context;
    const char *file;
    int line;
    CVStatus retval;

    Ch_VaStart(ap, varg);
    status = Ch_VaArg(ap, CVStatus);
    funcName = Ch_VaArg(ap, const char *);
    context = Ch_VaArg(ap, const char *);
    file = Ch_VaArg(ap, const char *);
    line = Ch_VaArg(ap, int);
    retval = cvGuiBoxReport(status, funcName, context, file, line);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvGetCallStack_chdl(void *varg) {
    va_list ap;
    CvStackRecord** stack;
    int* size;

    Ch_VaStart(ap, varg);
    stack = Ch_VaArg(ap, CvStackRecord**);
    size = Ch_VaArg(ap, int*);
    cvGetCallStack(stack, size);
    Ch_VaEnd(ap);
}

EXPORTCH void cvStartProfile_chdl(void *varg) {
    va_list ap;
    const char* call;
    const char* file;
    int line ;

    Ch_VaStart(ap, varg);
    call = Ch_VaArg(ap, const char*);
    file = Ch_VaArg(ap, const char*);
    line  = Ch_VaArg(ap, int);
    cvStartProfile(call, file, line );
    Ch_VaEnd(ap);
}

EXPORTCH void cvEndProfile_chdl(void *varg) {
    va_list ap;
    const char* file;
    int line;

    Ch_VaStart(ap, varg);
    file = Ch_VaArg(ap, const char*);
    line = Ch_VaArg(ap, int);
    cvEndProfile(file, line);
    Ch_VaEnd(ap);
}

/*
EXPORTCH void cvSetProfile_chdl(void *varg) {
    va_list ap;
    CvStartProfileFunc startProfile;
    CvEndProfileFunc endProfile;

    Ch_VaStart(ap, varg);
    startProfile = Ch_VaArg(ap, CvStartProfileFunc);
    endProfile = Ch_VaArg(ap, CvEndProfileFunc);
    cvSetProfile(startProfile, endProfile);
    Ch_VaEnd(ap);
}

*/
EXPORTCH void cvRemoveProfile_chdl(void *varg) {
    cvRemoveProfile();
}

/*
EXPORTCH void cvSetMemoryManager_chdl(void *varg) {
    va_list ap;
    CvAllocFunc allocFunc;
    CvFreeFunc freeFunc;

    Ch_VaStart(ap, varg);
    allocFunc = Ch_VaArg(ap, CvAllocFunc);
    freeFunc = Ch_VaArg(ap, CvFreeFunc);
    cvSetMemoryManager(allocFunc, freeFunc);
    Ch_VaEnd(ap);
}

*/
/*
EXPORTCH void cvSetIPLAllocators_chdl(void *varg) {
    va_list ap;
    Cv_iplCreateImageHeader createHeader;
    Cv_iplAllocateImageData allocateData;
    Cv_iplDeallocate deallocate;
    Cv_iplCreateROI createROI;
    Cv_iplCloneImage cloneImage;

    Ch_VaStart(ap, varg);
    createHeader = Ch_VaArg(ap, Cv_iplCreateImageHeader);
    allocateData = Ch_VaArg(ap, Cv_iplAllocateImageData);
    deallocate = Ch_VaArg(ap, Cv_iplDeallocate);
    createROI = Ch_VaArg(ap, Cv_iplCreateROI);
    cloneImage = Ch_VaArg(ap, Cv_iplCloneImage);
    cvSetIPLAllocators(createHeader, allocateData, deallocate, createROI, cloneImage);
    Ch_VaEnd(ap);
}

*/
EXPORTCH CvFileStorage* cvOpenFileStorage_chdl(void *varg) {
    va_list ap;
    const char* filename;
    CvMemStorage* storage;
    int flags ;
    CvFileStorage* retval;

    Ch_VaStart(ap, varg);
    filename = Ch_VaArg(ap, const char*);
    storage = Ch_VaArg(ap, CvMemStorage*);
    flags  = Ch_VaArg(ap, int);
    retval = cvOpenFileStorage(filename, storage, flags );
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvReleaseFileStorage_chdl(void *varg) {
    va_list ap;
    CvFileStorage** storage ;

    Ch_VaStart(ap, varg);
    storage  = Ch_VaArg(ap, CvFileStorage**);
    cvReleaseFileStorage(storage );
    Ch_VaEnd(ap);
}

EXPORTCH const char* cvAttrValue_chdl(void *varg) {
    va_list ap;
    const CvAttrList* attr;
    const char* attr_name ;
    const char* retval;

    Ch_VaStart(ap, varg);
    attr = Ch_VaArg(ap, const CvAttrList*);
    attr_name  = Ch_VaArg(ap, const char*);
    retval = cvAttrValue(attr, attr_name );
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvWrite_chdl(void *varg) {
    va_list ap;
    CvFileStorage* storage;
    const char* name;
    const void* struct_ptr;
    CvAttrList attributes;
    int flags;

    Ch_VaStart(ap, varg);
    storage = Ch_VaArg(ap, CvFileStorage*);
    name = Ch_VaArg(ap, const char*);
    struct_ptr = Ch_VaArg(ap, const void*);
    attributes = Ch_VaArg(ap, CvAttrList);
    flags = Ch_VaArg(ap, int);
    cvWrite(storage, name, struct_ptr, attributes, flags);
    Ch_VaEnd(ap);
}

EXPORTCH void cvStartWriteStruct_chdl(void *varg) {
    va_list ap;
    CvFileStorage* storage;
    const char* name;
    const char* type_name;
    const void* struct_ptr;
    CvAttrList attributes;

    Ch_VaStart(ap, varg);
    storage = Ch_VaArg(ap, CvFileStorage*);
    name = Ch_VaArg(ap, const char*);
    type_name = Ch_VaArg(ap, const char*);
    struct_ptr = Ch_VaArg(ap, const void*);
    attributes = Ch_VaArg(ap, CvAttrList);
    cvStartWriteStruct(storage, name, type_name, struct_ptr, attributes);
    Ch_VaEnd(ap);
}

EXPORTCH void cvEndWriteStruct_chdl(void *varg) {
    va_list ap;
    CvFileStorage* storage ;

    Ch_VaStart(ap, varg);
    storage  = Ch_VaArg(ap, CvFileStorage*);
    cvEndWriteStruct(storage );
    Ch_VaEnd(ap);
}

EXPORTCH void cvWriteElem_chdl(void *varg) {
    va_list ap;
    CvFileStorage* storage;
    const char* name;
    const char* elem_spec;
    const void* data_ptr;

    Ch_VaStart(ap, varg);
    storage = Ch_VaArg(ap, CvFileStorage*);
    name = Ch_VaArg(ap, const char*);
    elem_spec = Ch_VaArg(ap, const char*);
    data_ptr = Ch_VaArg(ap, const void*);
    cvWriteElem(storage, name, elem_spec, data_ptr);
    Ch_VaEnd(ap);
}

EXPORTCH CvFileNode* cvGetFileNode_chdl(void *varg) {
    va_list ap;
    CvFileStorage* storage;
    const char* name;
    CvFileNode* retval;

    Ch_VaStart(ap, varg);
    storage = Ch_VaArg(ap, CvFileStorage*);
    name = Ch_VaArg(ap, const char*);
    retval = cvGetFileNode(storage, name);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH const void* cvReadFileNode_chdl(void *varg) {
    va_list ap;
    CvFileStorage* storage;
    CvFileNode* node;
    CvAttrList** attr_list;
    const void* retval;

    Ch_VaStart(ap, varg);
    storage = Ch_VaArg(ap, CvFileStorage*);
    node = Ch_VaArg(ap, CvFileNode*);
    attr_list = Ch_VaArg(ap, CvAttrList**);
    retval = cvReadFileNode(storage, node, attr_list);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH int cvReadElem_chdl(void *varg) {
    va_list ap;
    CvFileStorage* storage;
    const char* name;
    void* data_ptr;
    int retval;

    Ch_VaStart(ap, varg);
    storage = Ch_VaArg(ap, CvFileStorage*);
    name = Ch_VaArg(ap, const char*);
    data_ptr = Ch_VaArg(ap, void*);
    retval = cvReadElem(storage, name, data_ptr);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CvTypeInfo* cvRegisterType_chdl(void *varg) {
    va_list ap;
    CvTypeInfo* info_data ;
    CvTypeInfo* retval;

    Ch_VaStart(ap, varg);
    info_data  = Ch_VaArg(ap, CvTypeInfo*);
    retval = cvRegisterType(info_data );
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvUnregisterType_chdl(void *varg) {
    va_list ap;
    const char* type_name ;

    Ch_VaStart(ap, varg);
    type_name  = Ch_VaArg(ap, const char*);
    cvUnregisterType(type_name );
    Ch_VaEnd(ap);
}

EXPORTCH CvTypeInfo* cvFirstType_chdl(void *varg) {
    CvTypeInfo* retval;
    retval = cvFirstType();
    return retval;
}

EXPORTCH CvTypeInfo* cvFindType_chdl(void *varg) {
    va_list ap;
    const char* type_name ;
    CvTypeInfo* retval;

    Ch_VaStart(ap, varg);
    type_name  = Ch_VaArg(ap, const char*);
    retval = cvFindType(type_name );
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH CvTypeInfo* cvTypeOf_chdl(void *varg) {
    va_list ap;
    const void* struct_ptr;
    CvTypeInfo* retval;

    Ch_VaStart(ap, varg);
    struct_ptr = Ch_VaArg(ap, const void*);
    retval = cvTypeOf(struct_ptr);
    Ch_VaEnd(ap);
    return retval;
}

EXPORTCH void cvRelease_chdl(void *varg) {
    va_list ap;
    void** struct_ptr;

    Ch_VaStart(ap, varg);
    struct_ptr = Ch_VaArg(ap, void**);
    cvRelease(struct_ptr);
    Ch_VaEnd(ap);
}

EXPORTCH void* cvClone_chdl(void *varg) {
    va_list ap;
    const void* struct_ptr;
    void* retval;

    Ch_VaStart(ap, varg);
    struct_ptr = Ch_VaArg(ap, const void*);
    retval = cvClone(struct_ptr);
    Ch_VaEnd(ap);
    return retval;
}
